from __future__ import annotations

from hashlib import sha256

import datasets

from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval

_LANGUAGES = {
    "as": ["asm-Beng"],
    "bn": ["ben-Beng"],
    "gu": ["guj-Gujr"],
    "hi": ["hin-Deva"],
    "kn": ["kan-Knda"],
    "ml": ["mal-Mlym"],
    "mr": ["mar-Deva"],
    "or": ["ory-Orya"],
    "pa": ["pan-Guru"],
    "ta": ["tam-Taml"],
    "te": ["tel-Telu"],
}


class IndicQARetrieval(MultilingualTask, AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="IndicQARetrieval",
        dataset={
            "path": "ai4bharat/IndicQA",
            "revision": "570d90ae4f7b64fe4fdd5f42fc9f9279b8c9fd9d",
            "trust_remote_code": True,
        },
        description="IndicQA is a manually curated cloze-style reading comprehension dataset that can be used for evaluating question-answering models in 11 Indic languages. It is repurposed retrieving relevant context for each question.",
        reference="https://arxiv.org/abs/2212.05409",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_10",
        date=("2022-08-01", "2022-12-20"),
        domains=["Web", "Written"],
        task_subtypes=[],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated and verified",
        bibtex_citation="""@article{doddapaneni2022towards,
  title     = {Towards Leaving No Indic Language Behind: Building Monolingual Corpora, Benchmark and Models for Indic Languages},
  author    = {Sumanth Doddapaneni and Rahul Aralikatte and Gowtham Ramesh and Shreyansh Goyal and Mitesh M. Khapra and Anoop Kunchukuttan and Pratyush Kumar},
  journal   = {Annual Meeting of the Association for Computational Linguistics},
  year      = {2022},
  doi       = {10.18653/v1/2023.acl-long.693}
}""",
        descriptive_stats={
            "n_samples": {"test": 18586},
            "avg_character_length": {
                "test": {
                    "as": {
                        "average_document_length": 1401.28,
                        "average_query_length": 56.60504201680672,
                        "num_documents": 250,
                        "num_queries": 1785,
                        "average_relevant_docs_per_query": 1.0016806722689076,
                    },
                    "bn": {
                        "average_document_length": 2196.012,
                        "average_query_length": 57.069239500567534,
                        "num_documents": 250,
                        "num_queries": 1762,
                        "average_relevant_docs_per_query": 1.0005675368898979,
                    },
                    "gu": {
                        "average_document_length": 960.4959677419355,
                        "average_query_length": 60.3712158808933,
                        "num_documents": 248,
                        "num_queries": 2015,
                        "average_relevant_docs_per_query": 1.0009925558312656,
                    },
                    "hi": {
                        "average_document_length": 2550.770114942529,
                        "average_query_length": 52.84909326424871,
                        "num_documents": 261,
                        "num_queries": 1544,
                        "average_relevant_docs_per_query": 1.0019430051813472,
                    },
                    "kn": {
                        "average_document_length": 882.7354085603113,
                        "average_query_length": 50.58734344100198,
                        "num_documents": 257,
                        "num_queries": 1517,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ml": {
                        "average_document_length": 2522.6437246963565,
                        "average_query_length": 75.93635790800252,
                        "num_documents": 247,
                        "num_queries": 1587,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "mr": {
                        "average_document_length": 1711.74,
                        "average_query_length": 58.785,
                        "num_documents": 250,
                        "num_queries": 1600,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "or": {
                        "average_document_length": 801.9206349206349,
                        "average_query_length": 55.072792362768496,
                        "num_documents": 252,
                        "num_queries": 1676,
                        "average_relevant_docs_per_query": 1.0011933174224343,
                    },
                    "pa": {
                        "average_document_length": 1423.5062240663901,
                        "average_query_length": 58.394925178919976,
                        "num_documents": 241,
                        "num_queries": 1537,
                        "average_relevant_docs_per_query": 1.0013012361743656,
                    },
                    "ta": {
                        "average_document_length": 2288.2608695652175,
                        "average_query_length": 54.06211869107044,
                        "num_documents": 253,
                        "num_queries": 1803,
                        "average_relevant_docs_per_query": 1.0005546311702718,
                    },
                    "te": {
                        "average_document_length": 2936.176,
                        "average_query_length": 67.00634371395617,
                        "num_documents": 250,
                        "num_queries": 1734,
                        "average_relevant_docs_per_query": 1.0,
                    },
                }
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        split = "test"
        queries = {lang: {split: {}} for lang in self.hf_subsets}
        corpus = {lang: {split: {}} for lang in self.hf_subsets}
        relevant_docs = {lang: {split: {}} for lang in self.hf_subsets}

        for lang in self.hf_subsets:
            data = datasets.load_dataset(
                name=f"indicqa.{lang}", **self.metadata_dict["dataset"]
            )[split]
            data = data.filter(lambda x: x["answers"]["text"] != "")

            question_ids = {
                question: sha256(question.encode("utf-8")).hexdigest()
                for question in set(data["question"])
            }
            context_ids = {
                context: sha256(context.encode("utf-8")).hexdigest()
                for context in set(data["context"])
            }

            for row in data:
                question = row["question"]
                context = row["context"]
                query_id = question_ids[question]
                queries[lang][split][query_id] = question

                doc_id = context_ids[context]
                corpus[lang][split][doc_id] = {"text": context}
                if query_id not in relevant_docs[lang][split]:
                    relevant_docs[lang][split][query_id] = {}
                relevant_docs[lang][split][query_id][doc_id] = 1

        self.corpus = datasets.DatasetDict(corpus)
        self.queries = datasets.DatasetDict(queries)
        self.relevant_docs = datasets.DatasetDict(relevant_docs)

        self.data_loaded = True



from __future__ import annotations

from collections import defaultdict

from datasets import DatasetDict, load_dataset

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


def load_retrieval_data(dataset_path, eval_splits):
    eval_split = eval_splits[0]
    corpus_dataset = load_dataset(dataset_path, "corpus")
    queries_dataset = load_dataset(dataset_path, "queries")
    qrels = load_dataset(dataset_path + "-qrels")[eval_split]

    corpus = {e["_id"]: {"text": e["text"]} for e in corpus_dataset["corpus"]}
    queries = {e["_id"]: e["text"] for e in queries_dataset["queries"]}
    relevant_docs = defaultdict(dict)
    for e in qrels:
        relevant_docs[e["query-id"]][e["corpus-id"]] = e["score"]

    corpus = DatasetDict({eval_split: corpus})
    queries = DatasetDict({eval_split: queries})
    relevant_docs = DatasetDict({eval_split: relevant_docs})
    return corpus, queries, relevant_docs


class GermanQuADRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="GermanQuAD-Retrieval",
        description="Context Retrieval for German Question Answering",
        reference="https://www.kaggle.com/datasets/GermanQuAD",
        dataset={
            "path": "mteb/germanquad-retrieval",
            "revision": "f5c87ae5a2e7a5106606314eef45255f03151bb3",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="mrr_at_5",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""misc{möller2021germanquad,
      title={GermanQuAD and GermanDPR: Improving Non-English Question Answering and Passage Retrieval}, 
      author={Timo Möller and Julian Risch and Malte Pietsch},
      year={2021},
      eprint={2104.12741},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "average_document_length": 1941.090717299578,
                    "average_query_length": 56.74773139745916,
                    "num_documents": 474,
                    "num_queries": 2204,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"], self.metadata_dict["eval_splits"]
        )
        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.AbsTaskClustering import AbsTaskClustering
from mteb.abstasks.AbsTaskClusteringFast import (
    AbsTaskClusteringFast,
    check_label_distribution,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class BiorxivClusteringP2PFast(AbsTaskClusteringFast):
    metadata = TaskMetadata(
        name="BiorxivClusteringP2P.v2",
        description="Clustering of titles+abstract from biorxiv across 26 categories.",
        reference="https://api.biorxiv.org/",
        dataset={
            "path": "mteb/biorxiv-clustering-p2p",
            "revision": "f5dbc242e11dd8e24def4c4268607a49e02946dc",
        },
        type="Clustering",
        category="p2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2021-01-01", "2022-05-10"),
        domains=["Academic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="https://www.biorxiv.org/content/about-biorxiv",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",
        descriptive_stats={
            "n_samples": {"test": 2151},
            "avg_character_length": {"test": 1664.0},
        },
    )

    def dataset_transform(self):
        for split in self.metadata.eval_splits:
            check_label_distribution(self.dataset[split])


class BiorxivClusteringP2P(AbsTaskClustering):
    superseded_by = "BiorxivClusteringP2P.v2"
    metadata = TaskMetadata(
        name="BiorxivClusteringP2P",
        description="Clustering of titles+abstract from biorxiv. Clustering of 10 sets, based on the main category.",
        reference="https://api.biorxiv.org/",
        dataset={
            "path": "mteb/biorxiv-clustering-p2p",
            "revision": "65b79d1d13f80053f67aca9498d9402c2d9f1f40",
        },
        type="Clustering",
        category="p2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2021-01-01", "2022-05-10"),
        domains=["Academic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="https://www.biorxiv.org/content/about-biorxiv",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",
        descriptive_stats={
            "n_samples": {"test": 75000},
            "avg_character_length": {"test": 1666.2},
        },
    )



"""test that the mteb.MTEB works as intended and that encoders are correctly called and passed the correct arguments."""

from __future__ import annotations

import logging
from pathlib import Path

import numpy as np
import pytest
from sentence_transformers import SentenceTransformer

import mteb
from mteb.benchmarks.benchmarks import Benchmark
from mteb.create_meta import generate_readme

from .mock_models import (
    MockNumpyEncoder,
    MockTorchbf16Encoder,
    MockTorchEncoder,
)
from .task_grid import MOCK_TASK_TEST_GRID

logging.basicConfig(level=logging.INFO)


@pytest.mark.parametrize("tasks", [MOCK_TASK_TEST_GRID])
@pytest.mark.parametrize("model", [MockNumpyEncoder()])
def test_mulitple_mteb_tasks(
    tasks: list[mteb.AbsTask], model: mteb.Encoder, tmp_path: Path
):
    """Test that multiple tasks can be run"""
    eval = mteb.MTEB(tasks=tasks)
    eval.run(model, output_folder=str(tmp_path), overwrite_results=True)

    # ensure that we can generate a readme from the output folder
    generate_readme(tmp_path)


@pytest.mark.parametrize("task", MOCK_TASK_TEST_GRID)
@pytest.mark.parametrize(
    "model",
    [
        MockNumpyEncoder(),
        MockTorchEncoder(),
        MockTorchbf16Encoder(),
    ],
)
def test_benchmark_encoders_on_task(task: str | mteb.AbsTask, model: mteb.Encoder):
    """Test that a task can be fetched and run using a variety of encoders"""
    if isinstance(task, str):
        tasks = mteb.get_tasks(tasks=[task])
    else:
        tasks = [task]

    eval = mteb.MTEB(tasks=tasks)
    eval.run(model, output_folder="tests/results", overwrite_results=True)


@pytest.mark.parametrize("task", MOCK_TASK_TEST_GRID[:1])
@pytest.mark.parametrize("model", [MockNumpyEncoder()])
def test_reload_results(task: str | mteb.AbsTask, model: mteb.Encoder, tmp_path: Path):
    """Test that when rerunning the results are reloaded correctly"""
    if isinstance(task, str):
        tasks = mteb.get_tasks(tasks=[task])
    else:
        tasks = [task]

    eval = mteb.MTEB(tasks=tasks)
    results = eval.run(model, output_folder=str(tmp_path), overwrite_results=True)

    assert isinstance(results, list)
    assert isinstance(results[0], mteb.MTEBResults)

    # reload the results
    results = eval.run(model, output_folder=str(tmp_path), overwrite_results=False)

    assert isinstance(results, list)
    assert isinstance(results[0], mteb.MTEBResults)


@pytest.mark.parametrize("task_name", MOCK_TASK_TEST_GRID)
def test_prompt_name_passed_to_all_encodes(task_name: str | mteb.AbsTask):
    """Test that all tasks correctly pass down the task_name to the encoder which supports it, and that the encoder which does not support it does not
    receive it.
    """
    _task_name = (
        task_name.metadata.name if isinstance(task_name, mteb.AbsTask) else task_name
    )

    class MockEncoderWithInstructions(mteb.Encoder):
        def encode(self, sentences, prompt_name: str | None = None, **kwargs):
            assert prompt_name == _task_name
            return np.zeros((len(sentences), 10))

    class EncoderWithoutInstructions(SentenceTransformer):
        def encode(self, sentences, **kwargs):
            assert "prompt_name" not in kwargs
            return super().encode(sentences, **kwargs)

    if isinstance(task_name, mteb.AbsTask):
        tasks = [task_name]
    else:
        tasks = mteb.get_tasks(tasks=[task_name])

    eval = mteb.MTEB(tasks=tasks)

    # Test that the task_name is passed down to the encoder
    model = MockEncoderWithInstructions()
    eval.run(model, output_folder="tests/results", overwrite_results=True)
    # Test that the task_name is not passed down to the encoder
    model = EncoderWithoutInstructions("average_word_embeddings_levy_dependency")
    assert model.prompts == {}, "The encoder should not have any prompts"
    eval.run(model, output_folder="tests/results", overwrite_results=True)


@pytest.mark.parametrize("task_name", MOCK_TASK_TEST_GRID)
def test_encode_kwargs_passed_to_all_encodes(task_name: str | mteb.AbsTask):
    """Test that all tasks correctly pass down the encode_kwargs to the encoder."""
    my_encode_kwargs = {"no_one_uses_this_args": "but_its_here"}

    class MockEncoderWithKwargs(mteb.Encoder):
        def encode(self, sentences, prompt_name: str | None = None, **kwargs):
            assert kwargs == my_encode_kwargs
            return np.zeros((len(sentences), 10))

    if isinstance(task_name, mteb.AbsTask):
        tasks = [task_name]
    else:
        tasks = mteb.get_tasks(tasks=[task_name])

    eval = mteb.MTEB(tasks=tasks)

    # Test that the task_name is passed down to the encoder
    model = MockEncoderWithKwargs()
    eval.run(
        model,
        output_folder="tests/results",
        overwrite_results=True,
        encode_kwargs=my_encode_kwargs,
    )


@pytest.mark.parametrize("model", [MockNumpyEncoder()])
def test_run_using_benchmark(model: mteb.Encoder):
    """Test that a benchmark object can be run using the MTEB class."""
    bench = Benchmark(
        name="test_bench", tasks=mteb.get_tasks(tasks=["STS12", "SummEval"])
    )

    eval = mteb.MTEB(tasks=bench)
    eval.run(
        model, output_folder="tests/results", overwrite_results=True
    )  # we just want to test that it runs


def test_benchmark_names_must_be_unique():
    import mteb.benchmarks.benchmarks as benchmark_module

    names = [
        inst.name
        for nam, inst in benchmark_module.__dict__.items()
        if isinstance(inst, Benchmark)
    ]
    assert len(names) == len(set(names))


@pytest.mark.parametrize("name", ["MTEB(eng)", "MTEB(rus)", "MTEB(Scandinavian)"])
def test_get_benchmark(name):
    benchmark = mteb.get_benchmark(benchmark_name=name)
    assert isinstance(benchmark, mteb.Benchmark)



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class HotpotQAPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="HotpotQA-PL",
        description="HotpotQA is a question answering dataset featuring natural, multi-hop questions, with strong supervision for supporting facts to enable more explainable question answering systems.",
        reference="https://hotpotqa.github.io/",
        dataset={
            "path": "clarin-knext/hotpotqa-pl",
            "revision": "a0bd479ac97b4ccb5bd6ce320c415d0bb4beb907",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2018-01-01", "2018-12-31"),  # best guess: based on publication date
        domains=["Web", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation="""@misc{wojtasik2024beirpl,
      title={BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language}, 
      author={Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
      year={2024},
      eprint={2305.19840},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "average_document_length": 292.26835882093405,
                    "average_query_length": 94.64064821066847,
                    "num_documents": 5233329,
                    "num_queries": 7405,
                    "average_relevant_docs_per_query": 2.0,
                }
            },
        },
    )


class HotpotQAPLHardNegatives(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="HotpotQA-PLHardNegatives",
        description="HotpotQA is a question answering dataset featuring natural, multi-hop questions, with strong supervision for supporting facts to enable more explainable question answering systems. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        reference="https://hotpotqa.github.io/",
        dataset={
            "path": "mteb/HotpotQA_PL_test_top_250_only_w_correct-v2",
            "revision": "0642cadffa3205c6b21c9af901fdffcd60d6f31e",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2018-01-01", "2018-12-31"),  # best guess: based on publication date
        domains=["Web", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation="""@misc{wojtasik2024beirpl,
      title={BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language}, 
      author={Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
      year={2024},
      eprint={2305.19840},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        descriptive_stats={
            "n_samples": {"test": 1000},
            "avg_character_length": {
                "test": {
                    "average_document_length": 438.3888210025661,
                    "average_query_length": 95.161,
                    "num_documents": 212774,
                    "num_queries": 1000,
                    "average_relevant_docs_per_query": 2.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class UkrFormalityClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="UkrFormalityClassification",
        description="""
        This dataset contains Ukrainian Formality Classification dataset obtained by
        trainslating English GYAFC data.
        English data source: https://aclanthology.org/N18-1012/
        Translation into Ukrainian language using model: https://huggingface.co/facebook/nllb-200-distilled-600M
        Additionally, the dataset was balanced, witha labels: 0 - informal, 1 - formal.
        """,
        dataset={
            "path": "ukr-detect/ukr-formality-dataset-translated-gyafc",
            "revision": "671d1e6bbf45a74ef21af351fd4ef7b32b7856f8",
        },
        reference="https://huggingface.co/datasets/ukr-detect/ukr-formality-dataset-translated-gyafc",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["train", "test"],
        eval_langs=["ukr-Cyrl"],
        main_score="accuracy",
        date=("2018-04-11", "2018-06-20"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="openrail++",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation="""@inproceedings{rao-tetreault-2018-dear,
        title = "Dear Sir or Madam, May {I} Introduce the {GYAFC} Dataset: Corpus, Benchmarks and Metrics for Formality Style Transfer",
        author = "Rao, Sudha  and
        Tetreault, Joel",
        booktitle = "Proceedings of the 2018 Conference of the North {A}merican Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 1 (Long Papers)",
        month = jun,
        year = "2018",
        publisher = "Association for Computational Linguistics",
        url = "https://aclanthology.org/N18-1012",
        }""",
        descriptive_stats={
            "n_samples": {"train": 2048, "test": 2048},
            "avg_character_length": {"train": 52.10, "test": 53.07},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("labels", "label")
        self.dataset = self.dataset.class_encode_column("label")
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["train", "test"]
        )



from __future__ import annotations

from mteb.abstasks.AbsTaskMultilabelClassification import (
    AbsTaskMultilabelClassification,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class BrazilianToxicTweetsClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="BrazilianToxicTweetsClassification",
        description="""
        ToLD-Br is the biggest dataset for toxic tweets in Brazilian Portuguese, crowdsourced by 42 annotators selected from
        a pool of 129 volunteers. Annotators were selected aiming to create a plural group in terms of demographics (ethnicity,
        sexual orientation, age, gender). Each tweet was labeled by three annotators in 6 possible categories: LGBTQ+phobia,
        Xenophobia, Obscene, Insult, Misogyny and Racism.
        """,
        reference="https://paperswithcode.com/dataset/told-br",
        dataset={
            "path": "JAugusto97/told-br",
            "revision": "fb4f11a5bc68b99891852d20f1ec074be6289768",
            "name": "multilabel",
            "trust_remote_code": True,
        },
        type="MultilabelClassification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["por-Latn"],
        main_score="accuracy",
        date=("2019-08-01", "2019-08-16"),
        domains=["Constructed", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=["brazilian"],
        sample_creation="found",
        bibtex_citation="""@article{DBLP:journals/corr/abs-2010-04543,
            author    = {Joao Augusto Leite and
                        Diego F. Silva and
                        Kalina Bontcheva and
                        Carolina Scarton},
            title     = {Toxic Language Detection in Social Media for Brazilian Portuguese:
                        New Dataset and Multilingual Analysis},
            journal   = {CoRR},
            volume    = {abs/2010.04543},
            year      = {2020},
            url       = {https://arxiv.org/abs/2010.04543},
            eprinttype = {arXiv},
            eprint    = {2010.04543},
            timestamp = {Tue, 15 Dec 2020 16:10:16 +0100},
            }""",
        descriptive_stats={
            "n_samples": {"test": 2048},
            "avg_character_length": {"test": 85.05},
        },
    )

    def dataset_transform(self):
        cols_ = ["homophobia", "obscene", "insult", "racism", "misogyny", "xenophobia"]
        n_size = len(self.dataset["train"])
        labels = [[] for _ in range(n_size)]
        for c in cols_:
            col_list = self.dataset["train"][c]
            for i in range(n_size):
                if col_list[i] > 0:
                    labels[i].append(c)
        self.dataset = self.dataset["train"].add_column("label", labels)
        del labels
        self.dataset = self.dataset.remove_columns(cols_)
        self.dataset = self.dataset.train_test_split(
            train_size=len(self.dataset) // 2,
            test_size=len(self.dataset) // 2,
            seed=self.seed,
        )
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["train"], n_samples=8192
        )
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"], n_samples=2048
        )



from __future__ import annotations

from functools import partial

from mteb.model_meta import ModelMeta, sentence_transformers_loader

mxbai_embed_large_v1 = ModelMeta(
    loader=partial(
        sentence_transformers_loader,
        model_name="mixedbread-ai/mxbai-embed-large-v1",
        revision="990580e27d329c7408b3741ecff85876e128e203",
        prompts={"query": "Represent this sentence for searching relevant passages: "},
    ),
    name="mixedbread-ai/mxbai-embed-large-v1",
    languages=["eng_Latn"],
    open_source=True,
    revision="990580e27d329c7408b3741ecff85876e128e203",
    release_date="2024-03-07",  # initial commit of hf model.
)



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class CzechSubjectivityClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="CzechSubjectivityClassification",
        description="An Czech dataset for subjectivity classification.",
        reference="https://arxiv.org/abs/2009.08712",
        dataset={
            "path": "pauli31/czech-subjectivity-dataset",
            "revision": "e387ddf167f3eba99936cff89909ed6264f17e1f",
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        date=("2022-04-01", "2022-04-01"),
        eval_splits=["validation", "test"],
        eval_langs=["ces-Latn"],
        main_score="accuracy",
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{priban-steinberger-2022-czech,
    title = "\{C\}zech Dataset for Cross-lingual Subjectivity Classification",
    author = "P{\v{r}}ib{\'a}{\v{n}}, Pavel  and
      Steinberger, Josef",
    booktitle = "Proceedings of the Thirteenth Language Resources and Evaluation Conference",
    month = jun,
    year = "2022",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2022.lrec-1.148",
    pages = "1381--1391",
}
""",
        descriptive_stats={
            "n_samples": {"validation": 500, "test": 2000},
            "avg_character_length": {"validation": 108.2, "test": 108.3},
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskPairClassification import AbsTaskPairClassification


class TwitterSemEval2015PC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="TwitterSemEval2015",
        dataset={
            "path": "mteb/twittersemeval2015-pairclassification",
            "revision": "70970daeab8776df92f5ea462b6173c0b46fd2d1",
        },
        description="Paraphrase-Pairs of Tweets from the SemEval 2015 workshop.",
        reference="https://alt.qcri.org/semeval2015/task1/",
        category="s2s",
        modalities=["text"],
        type="PairClassification",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="max_ap",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@inproceedings{xu-etal-2015-semeval,
        title = "{S}em{E}val-2015 Task 1: Paraphrase and Semantic Similarity in {T}witter ({PIT})",
        author = "Xu, Wei  and
        Callison-Burch, Chris  and
        Dolan, Bill",
        editor = "Nakov, Preslav  and
        Zesch, Torsten  and
        Cer, Daniel  and
        Jurgens, David",
        booktitle = "Proceedings of the 9th International Workshop on Semantic Evaluation ({S}em{E}val 2015)",
        month = jun,
        year = "2015",
        address = "Denver, Colorado",
        publisher = "Association for Computational Linguistics",
        url = "https://aclanthology.org/S15-2001",
        doi = "10.18653/v1/S15-2001",
        pages = "1--11",
    }""",
        descriptive_stats={
            "n_samples": {"test": 16777},
            "avg_character_length": {"test": 38.3},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("sent1", "sentence1")
        self.dataset = self.dataset.rename_column("sent2", "sentence2")



"""Mock models to be used for testing"""

from __future__ import annotations

from typing import Literal

import numpy as np
import torch
from numpy import ndarray
from sentence_transformers import SentenceTransformer
from torch import Tensor

import mteb


class MockNumpyEncoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return np.random.rand(len(sentences), 10)


class MockTorchEncoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return torch.randn(len(sentences), 10)


class MockTorchbf16Encoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return torch.randn(len(sentences), 10, dtype=torch.bfloat16)


class MockSentenceTransformer(SentenceTransformer):
    """A mock implementation of the SentenceTransformer intended to implement just the encode, method using the same arguments."""

    def __init__(self, *args, **kwargs):
        pass

    def encode(
        self,
        sentences: str | list[str],
        prompt_name: str | None = None,
        prompt: str | None = None,
        batch_size: int = 32,
        show_progress_bar: bool | None = None,
        output_value: Literal["sentence_embedding", "token_embeddings"]
        | None = "sentence_embedding",
        precision: Literal["float32", "int8", "uint8", "binary", "ubinary"] = "float32",
        convert_to_numpy: bool = True,
        convert_to_tensor: bool = False,
        device: str | None = None,
        normalize_embeddings: bool = False,
    ) -> list[Tensor] | ndarray | Tensor:
        return torch.randn(len(sentences), 10)



"""This script checks that all result have been run and writes a file: missing_tasks.txt with the missing tasks for each model"""

from __future__ import annotations

from typing import Iterable

import pandas as pd

import mteb

MODEL = str
REVISION = str


def results_to_dataframe(
    mteb_results: dict[MODEL, dict[REVISION, list[mteb.MTEBResults]]],
):
    data = []
    for model_name, revisions in mteb_results.items():
        for rev, tasks_results in revisions.items():
            for task_result in tasks_results:
                for split, scores in task_result.scores.items():
                    for score in scores:
                        data.append(
                            {
                                "model": model_name,
                                "revision": rev,
                                "task": task_result.task_name,
                                "split": split,
                                "hf_subset": score["hf_subset"],
                                "main_score": score["main_score"],
                            }
                        )
    return pd.DataFrame(data)


def filter_results(
    results,
    tasks: Iterable[mteb.AbsTask] | None = None,
    models: list[mteb.ModelMeta] | None = None,
):
    _results = {}

    iter_models = models if models is not None else results.keys()
    if tasks is not None:
        task_names = {t.metadata.name for t in tasks}

    for mdl in iter_models:
        if isinstance(mdl, mteb.ModelMeta):
            model_name = mdl.name
            revisions = (
                [mdl.revision] if mdl.revision is not None else results[model_name]
            )
        else:
            model_name = mdl
            revisions = results[model_name]

        _results[model_name] = {}

        if model_name not in results:
            continue

        for rev in revisions:
            _results[model_name][rev] = []

            if rev not in results[model_name]:
                continue

            tasks_results = results[model_name][rev]

            if tasks is not None:
                task_res = [r for r in tasks_results if r.task_name in task_names]
            else:
                task_res = tasks_results
            _results[model_name][rev] = task_res

    return _results


def normalize_results(results):
    for model_name, revisions in results.items():
        for rev, tasks_results in revisions.items():
            for task_result in tasks_results:
                try:
                    task_result.validate_and_filter_scores()
                except Exception:
                    print(
                        f"Error validating and filtering scores for {model_name} {rev} {task_result.task_name}. Some splits are missing"
                    )
                    # print(e)
    return results


model_names = [
    "sentence-transformers/all-MiniLM-L6-v2",
    "sentence-transformers/all-MiniLM-L12-v2",
    "sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    "sentence-transformers/all-mpnet-base-v2",
    "sentence-transformers/LaBSE",
    "intfloat/multilingual-e5-large-instruct",
    "intfloat/e5-mistral-7b-instruct",
    "GritLM/GritLM-7B",
    "GritLM/GritLM-8x7B",
    "intfloat/multilingual-e5-small",
    "intfloat/multilingual-e5-base",
    "intfloat/multilingual-e5-large",
]


tasks_which_should_be_there = mteb.get_tasks(
    task_types=[
        "BitextMining",
        "Classification",
        "MultilabelClassification",
        "PairClassification",
        "Reranking",
        "STS",
        "Summarization",
        "Clustering",
        "InstructionRetrieval",
        "Retrieval",
    ]
)

retrieval_to_be_downsampled = [
    "TopiOCQA",
    "MSMARCO-PL",
    "ClimateFEVER",
    "FEVER",
    "HotpotQA",
    "HotpotQA-PL",
    "DBPedia",
    "DBPedia-PL",
    "NeuCLIR2022Retrieval",
    "NeuCLIR2023Retrieval",
    "NeuCLIR2022Retrieval",
    "NeuCLIR2023Retrieval",
    "NQ",
    "NQ-PL",
    "NeuCLIR2022Retrieval",
    "NeuCLIR2023Retrieval",
    "MIRACLRetrieval",
    "RiaNewsRetrieval",
    "Quora-PL",
    "QuoraRetrieval",
]
not_include = ["MSMARCOv2"]

tasks_which_should_be_there = [
    t
    for t in tasks_which_should_be_there
    if t.metadata.name not in (retrieval_to_be_downsampled + not_include)
]

mteb_results = mteb.load_results()
models: list[mteb.ModelMeta] = [mteb.get_model_meta(name) for name in model_names]

# get missing revisions - Assuming we are using the latest revision
for model in models:
    if model.revision is None:
        print(f"Getting revision for {model.name}")
        encoder = model.load_model()
        model.revision = encoder.model_card_data.base_model_revision


mteb_results = filter_results(
    mteb_results, tasks=tasks_which_should_be_there, models=models
)

_mteb_results = normalize_results(mteb_results)
mteb_results = _mteb_results

# [t.task_name for t in mteb_results['GritLM/GritLM-7B']["13f00a0e36500c80ce12870ea513846a066004af"] if t.task_name == "SemRel24STS"]
# it is there

assert [
    len(revisions.keys()) == 1 for model, revisions in mteb_results.items()
], "Some models have more than one revision"

results_df = results_to_dataframe(mteb_results)

# results_df[results_df["task"] == "SemRel24STS"]  # still there

wide_table = results_df.pivot_table(
    index=["model", "revision"],
    columns=["task", "split", "hf_subset"],
    values="main_score",
)

# Find models which has NaN values
nans = wide_table[wide_table.isna().any(axis=1)]

# create list of model names x task names which is missing
t_names = {t.metadata.name for t in tasks_which_should_be_there}


sav_str = ""
for model in nans.index:
    print(f"Mising values for {model}:")
    sav_str += f"{model}:\n"
    nan_tasks = ""
    for task, split, lang in nans.columns:
        if task not in t_names:
            continue
        if task == nan_tasks:
            continue
        value = nans.loc[model, (task, split, lang)]
        if pd.isna(value):
            print(f"\t{task}")
            nan_tasks = task

            sav_str += f"\t{task}\n"


with open("missing_tasks.txt", "w") as f:
    f.write(sav_str)


# import mteb

# # running a model to ensure that the code works as expected
# task = mteb.get_task("FloresBitextMining")
# mdl = models[0].load_model()

# eval = mteb.MTEB(tasks = [task])
# results = eval.run(mdl)
# result = results[0]
# result.validate_and_filter_scores() # sucess


# len(set([s for s in task.metadata.hf_subsets_to_langscripts])) # 506

# # fetch from existing results
# grit_lm = [t for t in mteb_results["GritLM/GritLM-7B"]["13f00a0e36500c80ce12870ea513846a066004af"] if t.task_name == "FloresBitextMining"][0]

# grit_lm.validate_and_filter_scores() # fails
# # ValueError: Missing subsets {'ben_Beng-urd_Arab', 'sat_Olck-mal_Mlym', 'mar_Deva-snd_Deva', 'mar_Deva-san_Deva', 'mai_Deva-pan_Guru', 'sat_Olck-mni_Mtei', ...
# # KCE: have checked the results the keys are not present in the scores

# grit_lm.scores.keys() # ["test"]
# result.scores.keys() # ["test"]

# len(grit_lm.scores["devtest"]) # 20706
# len(result.scores["devtest"]) # 41412



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskInstructionRetrieval import AbsTaskInstructionRetrieval


class News21InstructionRetrieval(AbsTaskInstructionRetrieval):
    metadata = TaskMetadata(
        name="News21InstructionRetrieval",
        description="Measuring retrieval instruction following ability on News21 narratives.",
        reference="https://arxiv.org/abs/2403.15246",
        dataset={
            "path": "jhu-clsp/news21-instructions",
            "revision": "e0144086b45fe31ac125e9ac1a83b6a409bb6ca6",
        },
        type="InstructionRetrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="p-MRR",
        date=("2023-08-01", "2024-04-01"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{weller2024followir,
      title={FollowIR: Evaluating and Teaching Information Retrieval Models to Follow Instructions}, 
      author={Orion Weller and Benjamin Chang and Sean MacAvaney and Kyle Lo and Arman Cohan and Benjamin Van Durme and Dawn Lawrie and Luca Soldaini},
      year={2024},
      eprint={2403.15246},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        descriptive_stats={
            "n_samples": {"eng": 30953 * 2},
            "avg_character_length": {"eng": 2983.724665391969},
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskSTS import AbsTaskSTS

_EVAL_SPLIT = "test"


class STSES(AbsTaskSTS):
    metadata = TaskMetadata(
        name="STSES",
        dataset={
            "path": "PlanTL-GOB-ES/sts-es",
            "revision": "0912bb6c9393c76d62a7c5ee81c4c817ff47c9f4",
            "trust_remote_code": True,
        },
        description="Spanish test sets from SemEval-2014 (Agirre et al., 2014) and SemEval-2015 (Agirre et al., 2015)",
        reference="https://huggingface.co/datasets/PlanTL-GOB-ES/sts-es",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["spa-Latn"],
        main_score="cosine_spearman",
        date=None,
        domains=["Written"],
        task_subtypes=None,
        license="cc-by-4.0",
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@inproceedings{agirre2015semeval,
  title={Semeval-2015 task 2: Semantic textual similarity, english, spanish and pilot on interpretability},
  author={Agirre, Eneko and Banea, Carmen and Cardie, Claire and Cer, Daniel and Diab, Mona and Gonzalez-Agirre, Aitor and Guo, Weiwei and Lopez-Gazpio, Inigo and Maritxalar, Montse and Mihalcea, Rada and others},
  booktitle={Proceedings of the 9th international workshop on semantic evaluation (SemEval 2015)},
  pages={252--263},
  year={2015}
}


@inproceedings{agirre2014semeval,
  title={SemEval-2014 Task 10: Multilingual Semantic Textual Similarity.},
  author={Agirre, Eneko and Banea, Carmen and Cardie, Claire and Cer, Daniel M and Diab, Mona T and Gonzalez-Agirre, Aitor and Guo, Weiwei and Mihalcea, Rada and Rigau, German and Wiebe, Janyce},
  booktitle={SemEval@ COLING},
  pages={81--91},
  year={2014}
}
""",
        descriptive_stats={"n_samples": None, "avg_character_length": None},
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 5
        return metadata_dict

    def dataset_transform(self):
        data = self.dataset[_EVAL_SPLIT]
        data = data.add_column("score", [d["label"] for d in data])
        self.dataset = {_EVAL_SPLIT: data}



from __future__ import annotations

import datasets

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskReranking import AbsTaskReranking


class AlloprofReranking(AbsTaskReranking):
    metadata = TaskMetadata(
        name="AlloprofReranking",
        description="This dataset was provided by AlloProf, an organisation in Quebec, Canada offering resources and a help forum curated by a large number of teachers to students on all subjects taught from in primary and secondary school",
        reference="https://huggingface.co/datasets/antoinelb7/alloprof",
        dataset={
            "path": "lyon-nlp/mteb-fr-reranking-alloprof-s2p",
            "revision": "65393d0d7a08a10b4e348135e824f385d420b0fd",
        },
        type="Reranking",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fra-Latn"],
        main_score="map",
        date=("2020-01-01", "2023-04-14"),  # supposition
        domains=["Web", "Academic", "Written"],
        task_subtypes=None,
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=None,
        sample_creation="found",
        bibtex_citation="""@misc{lef23,
            doi = {10.48550/ARXIV.2302.07738},
            url = {https://arxiv.org/abs/2302.07738},
            author = {Lefebvre-Brossard, Antoine and Gazaille, Stephane and Desmarais, Michel C.},
            keywords = {Computation and Language (cs.CL), Information Retrieval (cs.IR), Machine Learning (cs.LG), FOS: Computer and information sciences, FOS: Computer and information sciences},
            title = {Alloprof: a new French question-answer education dataset and its use in an information retrieval case study},
            publisher = {arXiv},
            year = {2023},
            copyright = {Creative Commons Attribution Non Commercial Share Alike 4.0 International}
            }""",
        descriptive_stats={
            "n_samples": {"test": 2316, "train": 9264},
            "avg_character_length": None,
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            name="queries",
            **self.metadata_dict["dataset"],
            split=self.metadata.eval_splits[0],
        )
        documents = datasets.load_dataset(
            name="documents", **self.metadata_dict["dataset"], split="test"
        )
        # replace documents ids in positive and negative column by their respective texts
        doc_id2txt = dict(list(zip(documents["doc_id"], documents["text"])))

        self.dataset = self.dataset.map(
            lambda x: {
                "positive": [doc_id2txt[docid] for docid in x["positive"]],
                "negative": [doc_id2txt[docid] for docid in x["negative"]],
            }
        )
        self.dataset = datasets.DatasetDict({"test": self.dataset})

        self.dataset_transform()

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class AILAStatutes(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="AILAStatutes",
        description="This dataset is structured for the task of identifying the most relevant statutes for a given situation.",
        reference="https://zenodo.org/records/4063986",
        dataset={
            "path": "mteb/AILA_statutes",
            "revision": "ebfcd844eadd3d667efa3c57fc5c8c87f5c2867e",
        },
        type="Retrieval",
        category="p2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=["Legal", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=None,
        sample_creation="found",
        bibtex_citation="""@dataset{paheli_bhattacharya_2020_4063986,
  author       = {Paheli Bhattacharya and
                  Kripabandhu Ghosh and
                  Saptarshi Ghosh and
                  Arindam Pal and
                  Parth Mehta and
                  Arnab Bhattacharya and
                  Prasenjit Majumder},
  title        = {AILA 2019 Precedent \& Statute Retrieval Task},
  month        = oct,
  year         = 2020,
  publisher    = {Zenodo},
  doi          = {10.5281/zenodo.4063986},
  url          = {https://doi.org/10.5281/zenodo.4063986}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "average_document_length": 1973.6341463414635,
                    "average_query_length": 3038.42,
                    "num_documents": 82,
                    "num_queries": 50,
                    "average_relevant_docs_per_query": 4.34,
                }
            },
        },
    )



from __future__ import annotations

import datasets

from mteb.abstasks.AbsTaskPairClassification import AbsTaskPairClassification
from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGUAGES = {
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "fi": ["fin-Latn"],
    "fr": ["fra-Latn"],
    "ru": ["rus-Cyrl"],
    "sv": ["swe-Latn"],
}


class OpusparcusPC(AbsTaskPairClassification, MultilingualTask):
    metadata = TaskMetadata(
        name="OpusparcusPC",
        dataset={
            "path": "GEM/opusparcus",
            "revision": "9e9b1f8ef51616073f47f306f7f47dd91663f86a",
            "trust_remote_code": True,
        },
        description="Opusparcus is a paraphrase corpus for six European language: German, English, Finnish, French, Russian, and Swedish. The paraphrases consist of subtitles from movies and TV shows.",
        reference="https://gem-benchmark.com/data_cards/opusparcus",
        category="s2s",
        modalities=["text"],
        type="PairClassification",
        eval_splits=["test.full", "validation.full"],
        eval_langs=_LANGUAGES,
        main_score="max_ap",
        date=("2013-01-01", "2015-12-31"),
        domains=["Spoken", "Spoken"],
        task_subtypes=[],
        license="cc-by-nc-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation="""@misc{creutz2018open,
      title={Open Subtitles Paraphrase Corpus for Six Languages}, 
      author={Mathias Creutz},
      year={2018},
      eprint={1809.06142},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        descriptive_stats={
            "n_samples": {"validation": 10168, "test": 10210},
            "avg_character_length": {"validation": 24.4, "test": 23.8},
        },
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = {}
        for lang in self.hf_subsets:
            self.dataset[lang] = datasets.load_dataset(
                lang=lang,
                quality=100,
                **self.metadata_dict["dataset"],
            )
            self.dataset_transform(lang)
        self.data_loaded = True

    def dataset_transform(self, lang):
        for split in self.dataset[lang]:
            # Renaming features
            labels = self.dataset[lang][split]["annot_score"]
            sent1 = self.dataset[lang][split]["input"]
            sent2 = self.dataset[lang][split]["target"]
            new_dict = {}
            # Labels are a score between 1.0 and 4.0, and we need binary classification
            labels = [
                0 if label < 2.5 else 1 if label > 2.5 else 2.5 for label in labels
            ]
            # Get neutral label to delete them
            neutral = [i for i, val in enumerate(labels) if val == 2.5]
            for i in sorted(neutral, reverse=True):
                del labels[i]
                del sent1[i]
                del sent2[i]
            new_dict["labels"] = [labels]
            new_dict["sentence1"] = [sent1]
            new_dict["sentence2"] = [sent2]
            self.dataset[lang][split] = datasets.Dataset.from_dict(new_dict)



from __future__ import annotations

import os
from pathlib import Path

import mteb


def test_mteb_load_results():
    tests_path = Path(__file__).parent.parent / "mock_mteb_cache"

    os.environ["MTEB_CACHE"] = str(tests_path)

    results = mteb.load_results(download_latest=False)

    assert isinstance(results, dict)
    for model in results:
        assert isinstance(results[model], dict)
        for revision in results[model]:
            assert isinstance(results[model][revision], list)
            for result in results[model][revision]:
                assert isinstance(result, mteb.MTEBResults)

    known_model = "sentence-transformers/average_word_embeddings_levy_dependency"
    known_revision = "6d9c09a789ad5dd126b476323fccfeeafcd90509"
    assert known_model in results
    assert known_revision in results[known_model]



from __future__ import annotations

import pytest

from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.tasks.Retrieval.eng.NFCorpusRetrieval import NFCorpus


@pytest.mark.parametrize("task", [NFCorpus()])
def test_abstask_calculate_metadata_metrics(task: AbsTaskRetrieval):
    task.calculate_metadata_metrics()



from __future__ import annotations

from typing import Any

import datasets

from mteb.abstasks.AbsTaskBitextMining import AbsTaskBitextMining
from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGUAGES = [
    "aai_Latn",  # Apinayé
    "aak_Arab",  # Ankave
    "aau_Latn",  # Abau
    "aaz_Latn",  # Amarasi
    "abt_Latn",  # Ambulas
    "abx_Latn",  # Inabaknon
    "aby_Latn",  # Aneme Wake
    "acf_Latn",  # Saint Lucian Creole French
    "acr_Latn",  # Achi
    "acu_Latn",  # Achuar-Shiwiar
    "adz_Latn",  # Adzera
    "aer_Latn",  # Eastern Arrernte
    "aey_Latn",  # Amele
    "agd_Latn",  # Agarabi
    "agg_Latn",  # Angor
    "agm_Latn",  # Angaataha
    "agn_Latn",  # Agutaynen
    "agr_Latn",  # Aguaruna
    "agt_Latn",  # Central Cagayan Agta
    "agu_Latn",  # Aguacateco
    "aia_Latn",  # Arosi
    "aii_Syrc",  # Assyrian Neo-Aramaic
    "aka_Latn",  # Akan
    "ake_Latn",  # Akawaio
    "alp_Latn",  # Alune
    "alq_Latn",  # Algonquin
    "als_Latn",  # Tosk Albanian
    "aly_Latn",  # Alyawarr
    "ame_Latn",  # Yanesha'
    "amf_Latn",  # Hamer-Banna
    "amk_Latn",  # Ambai
    "amm_Latn",  # Ama (Papua New Guinea)
    "amn_Latn",  # Amanab
    "amo_Latn",  # Amo
    "amp_Latn",  # Alamblak
    "amr_Latn",  # Amarakaeri
    "amu_Latn",  # Amuzgo
    "amx_Latn",  # Anmatyerre
    "anh_Latn",  # Nend
    "anv_Latn",  # Denya
    "aoi_Latn",  # Anindilyakwa
    "aoj_Latn",  # Mufian
    "aom_Latn",  # Ömie
    "aon_Latn",  # Bumbita Arapesh
    "apb_Latn",  # Sa'a
    "ape_Latn",  # Bukiyip
    "apn_Latn",  # Apinayé
    "apr_Latn",  # Arop-Lokep
    "apu_Latn",  # Apurinã
    "apw_Latn",  # Western Apache
    "apz_Latn",  # Safeyoka
    "arb_Arab",  # Standard Arabic
    "are_Latn",  # Western Arrarnta
    "arl_Latn",  # Arabela
    "arn_Latn",  # Mapudungun
    "arp_Latn",  # Arapaho
    "asm_Beng",  # Assamese
    "aso_Latn",  # Dano
    "ata_Latn",  # Pele-Ata
    "atb_Latn",  # Zaiwa
    "atd_Latn",  # Ata Manobo
    "atg_Latn",  # Ivbie North-Okpela-Arhe
    "att_Latn",  # Pamplona Atta
    "auc_Latn",  # Waorani
    "aui_Latn",  # Anuki
    "auy_Latn",  # Awiyaana
    "avt_Latn",  # Au
    "awb_Latn",  # Awa (Papua New Guinea)
    "awk_Latn",  # Awabakal
    "awx_Latn",  # Awara
    "azb_Arab",  # South Azerbaijani
    "azg_Latn",  # San Pedro Amuzgos Amuzgo
    "azz_Latn",  # Highland Puebla Nahuatl
    "bao_Latn",  # Waimaha
    "bba_Latn",  # Baatonum
    "bbb_Latn",  # Barai
    "bbr_Latn",  # Girawa
    "bch_Latn",  # Bariai
    "bco_Latn",  # Kaluli
    "bdd_Latn",  # Bunama
    "bea_Latn",  # Beaver
    "bef_Latn",  # Benabena
    "bel_Cyrl",  # Belarusian
    "ben_Beng",  # Bengali
    "beo_Latn",  # Beami
    "beu_Latn",  # Blagar
    "bgs_Latn",  # Tagabawa
    "bgt_Latn",  # Bughotu
    "bhg_Latn",  # Binandere
    "bhl_Latn",  # Bimin
    "big_Latn",  # Biangai
    "bjk_Latn",  # Barok
    "bjp_Latn",  # Fanamaket
    "bjr_Latn",  # Binumarien
    "bjv_Latn",  # Bedjond
    "bjz_Latn",  # Baruga
    "bkd_Latn",  # Binukid
    "bki_Latn",  # Bikol
    "bkq_Latn",  # Bakairí
    "bkx_Latn",  # Baikeno
    "blw_Latn",  # Balangao
    "blz_Latn",  # Balantak
    "bmh_Latn",  # Kein
    "bmk_Latn",  # Ghayavi
    "bmr_Latn",  # Muinane
    "bmu_Latn",  # Somba-Siawari
    "bnp_Latn",  # Bola
    "boa_Latn",  # Bora
    "boj_Latn",  # Anjam
    "bon_Latn",  # Bine
    "box_Latn",  # Buamu
    "bpr_Latn",  # Koronadal Blaan
    "bps_Latn",  # Sarangani Blaan
    "bqc_Latn",  # Boko (Benin)
    "bqp_Latn",  # Busa
    "bre_Latn",  # Breton
    "bsj_Latn",  # Bangwinji
    "bsn_Latn",  # Barasana-Eduria
    "bsp_Latn",  # Baga Sitemu
    "bss_Latn",  # Akoose
    "buk_Latn",  # Bugawac
    "bus_Latn",  # Bokobaru
    "bvd_Latn",  # Baeggu
    "bvr_Latn",  # Burarra
    "bxh_Latn",  # Buhutu
    "byr_Latn",  # Baruya
    "byx_Latn",  # Qaqet
    "bzd_Latn",  # Bribri
    "bzh_Latn",  # Mapos Buang
    "bzj_Latn",  # Belize Kriol English
    "caa_Latn",  # Ch'orti'
    "cab_Latn",  # Garifuna
    "cac_Latn",  # Chuj
    "caf_Latn",  # Southern Carrier
    "cak_Latn",  # Kaqchikel
    "cao_Latn",  # Chácobo
    "cap_Latn",  # Chipaya
    "car_Latn",  # Galibi Carib
    "cav_Latn",  # Cavineña
    "cax_Latn",  # Chiquitano
    "cbc_Latn",  # Carapana
    "cbi_Latn",  # Chachi
    "cbk_Latn",  # Chavacano
    "cbr_Latn",  # Cashibo-Cacataibo
    "cbs_Latn",  # Cashinahua
    "cbt_Latn",  # Chayahuita
    "cbu_Latn",  # Candoshi-Shapra
    "cbv_Latn",  # Cacua
    "cco_Latn",  # Chamicuro
    "ceb_Latn",  # Cebuano
    "cek_Latn",  # Eastern Khumi Chin
    "ces_Latn",  # Czech
    "cgc_Latn",  # Kagayanen
    "cha_Latn",  # Chamorro
    "chd_Latn",  # Highland Oaxaca Chontal
    "chf_Latn",  # Tabasco Chontal
    "chk_Latn",  # Chuukese
    "chq_Latn",  # Quiotepec Chinantec
    "chz_Latn",  # Ozumacín Chinantec
    "cjo_Latn",  # Ashéninka Pajonal
    "cjv_Latn",  # Chuave
    "ckb_Arab",  # Central Kurdish
    "cle_Latn",  # Lealao Chinantec
    "clu_Latn",  # Caluyanun
    "cme_Latn",  # Cerma
    "cmn_Hans",  # Mandarin Chinese (Simplified)
    "cni_Latn",  # Asháninka
    "cnl_Latn",  # Lalana Chinantec
    "cnt_Latn",  # Tepetotutla Chinantec
    "cof_Latn",  # Colorado
    "con_Latn",  # Cofán
    "cop_Copt",  # Coptic
    "cot_Latn",  # Caquinte
    "cpa_Latn",  # Palantla Chinantec
    "cpb_Latn",  # Ucayali-Yurúa Ashéninka
    "cpc_Latn",  # Ajyíninka Apurucayali
    "cpu_Latn",  # Pichis Ashéninka
    "cpy_Latn",  # South Ucayali Ashéninka
    "crn_Latn",  # El Nayar Cora
    "crx_Latn",  # Carrier
    "cso_Latn",  # Sochiapam Chinantec
    "csy_Latn",  # Siyin Chin
    "cta_Latn",  # Tataltepec Chatino
    "cth_Latn",  # Thaiphum Chin
    "ctp_Latn",  # Western Highland Chatino
    "ctu_Latn",  # Chol
    "cub_Latn",  # Cubeo
    "cuc_Latn",  # Usila Chinantec
    "cui_Latn",  # Cuiba
    "cuk_Latn",  # San Blas Kuna
    "cut_Latn",  # Teutila Cuicatec
    "cux_Latn",  # Tepeuxila Cuicatec
    "cwe_Latn",  # Kwere
    "cya_Latn",  # Nopala Chatino
    "daa_Latn",  # Dangaléat
    "dad_Latn",  # Marik
    "dah_Latn",  # Gwahatike
    "dan_Latn",  # Danish
    "ded_Latn",  # Dedua
    "deu_Latn",  # German
    "dgc_Latn",  # Agta
    "dgr_Latn",  # Dogrib
    "dgz_Latn",  # Daga
    "dhg_Latn",  # Dhangu
    "dif_Latn",  # Dieri
    "dik_Latn",  # Southwestern Dinka
    "dji_Latn",  # Djinang
    "djk_Latn",  # Aukan
    "djr_Latn",  # Djambarrpuyngu
    "dob_Latn",  # Dobu
    "dop_Latn",  # Lukpa
    "dov_Latn",  # Dombe
    "dwr_Latn",  # Dawro
    "dww_Latn",  # Dawawa
    "dwy_Latn",  # Dhuwaya
    "ebk_Latn",  # Eastern Bontok
    "eko_Latn",  # Kota (Gabon)
    "emi_Latn",  # Mussau-Emira
    "emp_Latn",  # Northern Emberá
    "enq_Latn",  # Enga
    "epo_Latn",  # Esperanto
    "eri_Latn",  # Ogea
    "ese_Latn",  # Ese Ejja
    "esk_Latn",  # Northwest Alaska Inupiatun
    "etr_Latn",  # Edolo
    "ewe_Latn",  # Ewe
    "faa_Latn",  # Fasu
    "fai_Latn",  # Faiwol
    "far_Latn",  # Fataleka
    "ffm_Latn",  # Maasina Fulfulde
    "for_Latn",  # Fore
    "fra_Latn",  # French
    "fue_Latn",  # Borgu Fulfulde
    "fuf_Latn",  # Pular
    "fuh_Latn",  # Western Niger Fulfulde
    "gah_Latn",  # Alekano
    "gai_Latn",  # Borei
    "gam_Latn",  # Kandawo
    "gaw_Latn",  # Nobonob
    "gdn_Latn",  # Umanakaina
    "gdr_Latn",  # Wipi
    "geb_Latn",  # Kire
    "gfk_Latn",  # Patpatar
    "ghs_Latn",  # Guhu-Samane
    "glk_Arab",  # Gilaki
    "gmv_Latn",  # Gamo
    "gng_Latn",  # Ngangam
    "gnn_Latn",  # Gumatj
    "gnw_Latn",  # Western Bolivian Guaraní
    "gof_Latn",  # Gofa
    "grc_Grek",  # Ancient Greek
    "gub_Latn",  # Guajajára
    "guh_Latn",  # Guahibo
    "gui_Latn",  # Eastern Bolivian Guaraní
    "guj_Gujr",  # Gujarati
    "gul_Latn",  # Sea Island Creole English
    "gum_Latn",  # Guambiano
    "gun_Latn",  # Mbyá Guaraní
    "guo_Latn",  # Guayabero
    "gup_Latn",  # Gunwinggu
    "gux_Latn",  # Gourmanchéma
    "gvc_Latn",  # Guanano
    "gvf_Latn",  # Golin
    "gvn_Latn",  # Kuku-Yalanji
    "gvs_Latn",  # Gumawana
    "gwi_Latn",  # Gwichʼin
    "gym_Latn",  # Ngäbere
    "gyr_Latn",  # Guarayu
    "hat_Latn",  # Haitian Creole
    "hau_Latn",  # Hausa
    "haw_Latn",  # Hawaiian
    "hbo_Hebr",  # Ancient Hebrew
    "hch_Latn",  # Huichol
    "heb_Hebr",  # Hebrew
    "heg_Latn",  # Helong
    "hin_Deva",  # Hindi
    "hix_Latn",  # Hoocąk
    "hla_Latn",  # Halia
    "hlt_Latn",  # Matu Chin
    "hmo_Latn",  # Hiri Motu
    "hns_Latn",  # Caribbean Hindustani
    "hop_Latn",  # Hopi
    "hot_Latn",  # Hote
    "hrv_Latn",  # Croatian
    "hto_Latn",  # Minica Huitoto
    "hub_Latn",  # Huambisa
    "hui_Latn",  # Huli
    "hun_Latn",  # Hungarian
    "hus_Latn",  # Huastec
    "huu_Latn",  # Huitoto, Murui
    "huv_Latn",  # San Mateo Del Mar Huave
    "hvn_Latn",  # Sabu
    "ian_Latn",  # Iatmul
    "ign_Latn",  # Ignaciano
    "ikk_Latn",  # Ika
    "ikw_Latn",  # Ikwere
    "ilo_Latn",  # Iloko
    "imo_Latn",  # Imbongu
    "inb_Latn",  # Inga
    "ind_Latn",  # Indonesian
    "ino_Latn",  # Inoke-Yate
    "iou_Latn",  # Tuma-Irumu
    "ipi_Latn",  # Ipili
    "isn_Latn",  # Isanzu
    "ita_Latn",  # Italian
    "iws_Latn",  # Sepik Iwam
    "ixl_Latn",  # Ixil
    "jac_Latn",  # Popti'
    "jae_Latn",  # Yabem
    "jao_Latn",  # Yanyuwa
    "jic_Latn",  # Tol
    "jid_Latn",  # Bu
    "jiv_Latn",  # Shuar
    "jni_Latn",  # Janji
    "jpn_Jpan",  # Japanese
    "jvn_Latn",  # Caribbean Javanese
    "kan_Knda",  # Kannada
    "kaq_Latn",  # Capanahua
    "kbc_Latn",  # Kadiwéu
    "kbh_Latn",  # Camsá
    "kbm_Latn",  # Iwal
    "kbq_Latn",  # Kamano
    "kdc_Latn",  # Kutu
    "kde_Latn",  # Makonde
    "kdl_Latn",  # Tsikimba
    "kek_Latn",  # Kekchí
    "ken_Latn",  # Kenyang
    "kew_Latn",  # West Kewa
    "kgf_Latn",  # Kube
    "kgk_Latn",  # Kagoma
    "kgp_Latn",  # Kaingang
    "khs_Latn",  # Kasua
    "khz_Latn",  # Keapara
    "kik_Latn",  # Kikuyu
    "kiw_Latn",  # Northeast Kiwai
    "kiz_Latn",  # Kisi
    "kje_Latn",  # Kisar
    "kjs_Latn",  # East Kewa
    "kkc_Latn",  # Odoodee
    "kkl_Latn",  # Kokota
    "klt_Latn",  # Nukna
    "klv_Latn",  # Maskelynes
    "kmg_Latn",  # Kâte
    "kmh_Latn",  # Kalam
    "kmk_Latn",  # Limos Kalinga
    "kmo_Latn",  # Kwoma
    "kms_Latn",  # Kamasau
    "kmu_Latn",  # Kanite
    "kne_Latn",  # Kankanaey
    "knf_Latn",  # Mankanya
    "knj_Latn",  # Western Kanjobal
    "knv_Latn",  # Tabo
    "kos_Latn",  # Kosraean
    "kpf_Latn",  # Komba
    "kpg_Latn",  # Kapingamarangi
    "kpj_Latn",  # Karajá
    "kpr_Latn",  # Korafe-Yegha
    "kpw_Latn",  # Kobon
    "kpx_Latn",  # Mountain Koiali
    "kqa_Latn",  # Mum
    "kqc_Latn",  # Doromu-Koki
    "kqf_Latn",  # Kakabai
    "kql_Latn",  # Kyenele
    "kqw_Latn",  # Kandas
    "ksd_Latn",  # Kuanua
    "ksj_Latn",  # Uare
    "ksr_Latn",  # Borong
    "ktm_Latn",  # Kotan
    "kto_Latn",  # Kuot
    "kud_Latn",  # 'Auhelawa
    "kue_Latn",  # Kuman
    "kup_Latn",  # Kunimaipa
    "kvg_Latn",  # Kuria
    "kvn_Latn",  # Border Kuna
    "kwd_Latn",  # Kwaio
    "kwf_Latn",  # Kwara'ae
    "kwi_Latn",  # Awa-Cuaiquer
    "kwj_Latn",  # Kwanga
    "kyc_Latn",  # Kyaka
    "kyf_Latn",  # Kaiy
    "kyg_Latn",  # Keyagana
    "kyq_Latn",  # Kenga
    "kyz_Latn",  # Kayabí
    "kze_Latn",  # Kosena
    "lac_Latn",  # Lacandon
    "lat_Latn",  # Latin
    "lbb_Latn",  # Label
    "lbk_Latn",  # Central Bontok
    "lcm_Latn",  # Tungag
    "leu_Latn",  # Kara (Papua New Guinea)
    "lex_Latn",  # Luang
    "lgl_Latn",  # Wala
    "lid_Latn",  # Nyindrou
    "lif_Deva",  # Limbu
    "lin_Latn",  # Lingala
    "lit_Latn",  # Lithuanian
    "llg_Latn",  # Lole
    "lug_Latn",  # Ganda
    "luo_Latn",  # Luo (Kenya and Tanzania)
    "lww_Latn",  # Lewo Eleng
    "maa_Latn",  # San Jerónimo Tecóatl Mazatec
    "maj_Latn",  # Jalapa De Díaz Mazatec
    "mal_Mlym",  # Malayalam
    "mam_Latn",  # Mam
    "maq_Latn",  # Chiquihuitlán Mazatec
    "mar_Deva",  # Marathi
    "mau_Latn",  # Maumere
    "mav_Latn",  # Sateré-Mawé
    "maz_Latn",  # Central Mazahua
    "mbb_Latn",  # Manobo
    "mbc_Latn",  # Macushi
    "mbh_Latn",  # Mangseng
    "mbj_Latn",  # Nadëb
    "mbl_Latn",  # Maxakalí
    "mbs_Latn",  # Sarangani Manobo
    "mbt_Latn",  # Matigsalug Manobo
    "mca_Latn",  # Maca
    "mcb_Latn",  # Macushi
    "mcd_Latn",  # Sharanahua
    "mcf_Latn",  # Matsés
    "mco_Latn",  # Mixe, Coatlán
    "mcp_Latn",  # Maka (Chad)
    "mcq_Latn",  # Ese
    "mcr_Latn",  # Menya
    "mdy_Latn",  # Male (Ethiopia)
    "med_Latn",  # Melpa
    "mee_Latn",  # Mundurukú
    "mek_Latn",  # Mekeo
    "meq_Latn",  # Merey
    "met_Latn",  # Mato
    "meu_Latn",  # Motu
    "mgc_Latn",  # Morokodo
    "mgh_Latn",  # Makhuwa-Meetto
    "mgw_Latn",  # Matumbi
    "mhl_Latn",  # Mauwake
    "mib_Latn",  # Mixtepec Mixtec
    "mic_Latn",  # Mi'kmaq
    "mie_Latn",  # Mixtec, Ocotepec
    "mig_Latn",  # San Miguel El Grande Mixtec
    "mih_Latn",  # Mixtec, Chayuco
    "mil_Latn",  # Peñoles Mixtec
    "mio_Latn",  # Mixtec, Pinotepa Nacional
    "mir_Latn",  # Mixtec, Isthmus
    "mit_Latn",  # Mixtec, Southern Puebla
    "miz_Latn",  # Mixtec, Coatzospan
    "mjc_Latn",  # Mixtec, San Juan Colorado
    "mkj_Latn",  # Mokilko
    "mkl_Latn",  # Mokole
    "mkn_Latn",  # Kupang Malay
    "mks_Latn",  # Mixtec, Silacayoapan
    "mle_Latn",  # Manambu
    "mlh_Latn",  # Mape
    "mlp_Latn",  # Bargam
    "mmo_Latn",  # Mangga Buang
    "mmx_Latn",  # Madak
    "mna_Latn",  # Mbula
    "mop_Latn",  # Mopán Maya
    "mox_Latn",  # Molima
    "mph_Latn",  # Maung
    "mpj_Latn",  # Martu Wangka
    "mpm_Latn",  # Yosondúa Mixtec
    "mpp_Latn",  # Migabac
    "mps_Latn",  # Dadibi
    "mpt_Latn",  # Mian
    "mpx_Latn",  # Misima-Panaeati
    "mqb_Latn",  # Mbuko
    "mqj_Latn",  # Mamasa
    "msb_Latn",  # Masbatenyo
    "msc_Latn",  # Sankaran Maninka
    "msk_Latn",  # Mansaka
    "msm_Latn",  # Agusan Manobo
    "msy_Latn",  # Aruamu
    "mti_Latn",  # Maiwa (Papua New Guinea)
    "mto_Latn",  # Totontepec Mixe
    "mux_Latn",  # Bo-Ung
    "muy_Latn",  # Muyang
    "mva_Latn",  # Manam
    "mvn_Latn",  # Minaveha
    "mwc_Latn",  # Are
    "mwe_Latn",  # Mwera (Chimwera)
    "mwf_Latn",  # Murrinh-Patha
    "mwp_Latn",  # Kala Lagaw Ya
    "mxb_Latn",  # Tezoatlán Mixtec
    "mxp_Latn",  # Tlahuitoltepec Mixe
    "mxq_Latn",  # Juquila Mixe
    "mxt_Latn",  # Jamiltepec Mixtec
    "mya_Latn",  # Muya
    "myk_Latn",  # Mamara Senoufo
    "myu_Latn",  # Mundurukú
    "myw_Latn",  # Muyuw
    "myy_Latn",  # Macuna
    "mzz_Latn",  # Masimasi
    "nab_Latn",  # Southern Nambikuára
    "naf_Latn",  # Nabak
    "nak_Latn",  # Nakanai
    "nas_Latn",  # Naasioi
    "nbq_Latn",  # Nggem
    "nca_Latn",  # Iyo
    "nch_Latn",  # Central Huasteca Nahuatl
    "ncj_Latn",  # Northern Puebla Nahuatl
    "ncl_Latn",  # Michoacán Nahuatl
    "ncu_Latn",  # Chumburung
    "ndg_Latn",  # Ndengereko
    "ndj_Latn",  # Ndjem
    "nfa_Latn",  # Dhao
    "ngp_Latn",  # Ngulu
    "ngu_Latn",  # Guerrero Nahuatl
    "nhe_Latn",  # Eastern Huasteca Nahuatl
    "nhg_Latn",  # Tetelcingo Nahuatl
    "nhi_Latn",  # Zacatlán-Ahuacatlán-Tepetzintla Nahuatl
    "nho_Latn",  # Takuu
    "nhr_Latn",  # Naro
    "nhu_Latn",  # Noone
    "nhw_Latn",  # Western Huasteca Nahuatl
    "nhy_Latn",  # Northern Oaxaca Nahuatl
    "nif_Latn",  # Nek
    "nii_Latn",  # Nii
    "nin_Latn",  # Ninia Yali
    "nko_Latn",  # Nkonya
    "nld_Latn",  # Dutch
    "nlg_Latn",  # Gela
    "nna_Latn",  # North Ndebele
    "nnq_Latn",  # Ngindo
    "noa_Latn",  # Woun Meu
    "nop_Latn",  # Numanggang
    "not_Latn",  # Nomatsiguenga
    "nou_Latn",  # Ewage-Notu
    "npi_Deva",  # Nepali (individual language)
    "npl_Latn",  # Southeastern Puebla Nahuatl
    "nsn_Latn",  # Nehan
    "nss_Latn",  # Nali
    "ntj_Latn",  # Ngaanyatjarra
    "ntp_Latn",  # Tepehuan
    "ntu_Latn",  # Natügu
    "nuy_Latn",  # Nunggubuyu
    "nvm_Latn",  # Namiae
    "nwi_Latn",  # Southwest Tanna
    "nya_Latn",  # Nyanja
    "nys_Latn",  # Nyungar
    "nyu_Latn",  # Nyungwe
    "obo_Latn",  # Obo Manobo
    "okv_Latn",  # Orokaiva
    "omw_Latn",  # South Tairora
    "ong_Latn",  # Olo
    "ons_Latn",  # Ono
    "ood_Latn",  # Tohono O'odham
    "opm_Latn",  # Oksapmin
    "ory_Orya",  # Odia (Oriya)
    "ote_Latn",  # Mezquital Otomi
    "otm_Latn",  # Eastern Highland Otomi
    "otn_Latn",  # Tenango Otomi
    "otq_Latn",  # Querétaro Otomi
    "ots_Latn",  # Estado de México Otomi
    "pab_Latn",  # Parecís
    "pad_Latn",  # Paumarí
    "pah_Latn",  # Tenharim
    "pan_Guru",  # Panjabi
    "pao_Latn",  # Northern Paiute
    "pes_Arab",  # Iranian Persian
    "pib_Latn",  # Yine
    "pio_Latn",  # Piapoco
    "pir_Latn",  # Piratapuyo
    "piu_Latn",  # Pintupi-Luritja
    "pjt_Latn",  # Pitjantjatjara
    "pls_Latn",  # San Marcos Tlalcoyalco Popoloca
    "plu_Latn",  # Palikur
    "pma_Latn",  # Paama
    "poe_Latn",  # San Juan Atzingo Popoloca
    "poh_Latn",  # Poqomchi'
    "poi_Latn",  # Highland Popoluca
    "pol_Latn",  # Polish
    "pon_Latn",  # Pohnpeian
    "por_Latn",  # Portuguese
    "poy_Latn",  # Pogolo
    "ppo_Latn",  # Folopa
    "prf_Latn",  # Paranan
    "pri_Latn",  # Paicî
    "ptp_Latn",  # Patep
    "ptu_Latn",  # Bambam
    "pwg_Latn",  # Gapapaiwa
    "qub_Latn",  # Huallaga Huánuco Quechua
    "quc_Latn",  # K'iche'
    "quf_Latn",  # Lambayeque Quechua
    "quh_Latn",  # South Bolivian Quechua
    "qul_Latn",  # North Bolivian Quechua
    "qup_Latn",  # Southern Pastaza Quechua
    "qvc_Latn",  # Cajamarca Quechua
    "qve_Latn",  # Eastern Apurímac Quechua
    "qvh_Latn",  # Huamalíes-Dos de Mayo Huánuco Quechua
    "qvm_Latn",  # Margos-Yarowilca-Lauricocha Quechua
    "qvn_Latn",  # North Junín Quechua
    "qvs_Latn",  # San Martín Quechua
    "qvw_Latn",  # Huaylla Wanca Quechua
    "qvz_Latn",  # Northern Pastaza Quichua
    "qwh_Latn",  # Huaylas Ancash Quechua
    "qxh_Latn",  # Panao Huánuco Quechua
    "qxn_Latn",  # Northern Conchucos Ancash Quechua
    "qxo_Latn",  # Southern Conchucos Ancash Quechua
    "rai_Latn",  # Ramoaaina
    "reg_Latn",  # Kara (Papua New Guinea)
    "rgu_Latn",  # Ringgou
    "rkb_Latn",  # Rikbaktsa
    "rmc_Latn",  # Carpathian Romani
    "rmy_Latn",  # Vlax Romani
    "ron_Latn",  # Romanian
    "roo_Latn",  # Rotokas
    "rop_Latn",  # Kriol
    "row_Latn",  # Dela-Oenale
    "rro_Latn",  # Waima
    "ruf_Latn",  # Luguru
    "rug_Latn",  # Roviana
    "rus_Cyrl",  # Russian
    "rwo_Latn",  # Rawa
    "sab_Latn",  # Buglere
    "san_Latn",  # Sanskrit
    "sbe_Latn",  # Saliba
    "sbk_Latn",  # Safwa
    "sbs_Latn",  # Subiya
    "seh_Latn",  # Sena
    "sey_Latn",  # Secoya
    "sgb_Latn",  # Ayta Mag-antsi
    "sgz_Latn",  # Sursurunga
    "shj_Latn",  # Shatt
    "shp_Latn",  # Shipibo-Conibo
    "sim_Latn",  # Mende (Papua New Guinea)
    "sja_Latn",  # Epena
    "sll_Latn",  # Salt-Yui
    "smk_Latn",  # Bolinao
    "snc_Latn",  # Sinaugoro
    "snn_Latn",  # Siona
    "snp_Latn",  # Siane
    "snx_Latn",  # Sam
    "sny_Latn",  # Saniyo-Hiyewe
    "som_Latn",  # Somali
    "soq_Latn",  # Kanasi
    "soy_Latn",  # Miyobe
    "spa_Latn",  # Spanish
    "spl_Latn",  # Selepet
    "spm_Latn",  # Akukem
    "spp_Latn",  # Supyire Senoufo
    "sps_Latn",  # Saposa
    "spy_Latn",  # Sabaot
    "sri_Latn",  # Siriano
    "srm_Latn",  # Saramaccan
    "srn_Latn",  # Sranan Tongo
    "srp_Latn",  # Serbian
    "srq_Latn",  # Sirionó
    "ssd_Latn",  # Siroi
    "ssg_Latn",  # Seimat
    "ssx_Latn",  # Samberigi
    "stp_Latn",  # Southeastern Tepehuan
    "sua_Latn",  # Sulka
    "sue_Latn",  # Suena
    "sus_Arab",  # Susu
    "suz_Latn",  # Sunwar
    "swe_Latn",  # Swedish
    "swh_Latn",  # Swahili (Kiswahili)
    "swp_Latn",  # Suau
    "sxb_Latn",  # Suba
    "tac_Latn",  # Lowland Tarahumara
    "taj_Deva",  # Tajik
    "tam_Taml",  # Tamil
    "tav_Latn",  # Tatuyo
    "taw_Latn",  # Tai
    "tbc_Latn",  # Takia
    "tbf_Latn",  # Mandara
    "tbg_Latn",  # North Tairora
    "tbo_Latn",  # Tawala
    "tbz_Latn",  # Ditammari
    "tca_Latn",  # Ticuna
    "tcs_Latn",  # Torres Strait Creole
    "tcz_Latn",  # Thado Chin
    "tdt_Latn",  # Tetun Dili
    "tee_Latn",  # Huehuetla Tepehua
    "tel_Telu",  # Telugu
    "ter_Latn",  # Terêna
    "tet_Latn",  # Tetum
    "tew_Latn",  # Tewa (USA)
    "tfr_Latn",  # Me'phaa
    "tgk_Cyrl",  # Tajik
    "tgl_Latn",  # Tagalog
    "tgo_Latn",  # Sudest
    "tgp_Latn",  # Tangoa
    "tha_Thai",  # Thai
    "tif_Latn",  # Tifal
    "tim_Latn",  # Timbe
    "tiw_Latn",  # Tiwi
    "tiy_Latn",  # Tiruray
    "tke_Latn",  # Tsikimba
    "tku_Latn",  # Upper Necaxa Totonac
    "tlf_Latn",  # Telefol
    "tmd_Latn",  # Haruai
    "tna_Latn",  # Tacana
    "tnc_Latn",  # Tanimuca-Retuarã
    "tnk_Latn",  # Kwamera
    "tnn_Latn",  # North Tanna
    "tnp_Latn",  # Whitesands
    "toc_Latn",  # Coyutla Totonac
    "tod_Latn",  # Toma
    "tof_Latn",  # Columbia-Wenatchi
    "toj_Latn",  # Tojolabal
    "ton_Latn",  # Tonga (Tonga Islands)
    "too_Latn",  # Xicotepec De Juárez Totonac
    "top_Latn",  # Papantla Totonac
    "tos_Latn",  # Highland Totonac
    "tpa_Latn",  # Taupota
    "tpi_Latn",  # Tok Pisin
    "tpt_Latn",  # Tlachichilco Tepehua
    "tpz_Latn",  # Tinputz
    "trc_Latn",  # Copala Triqui
    "tsw_Latn",  # Tsishingini
    "ttc_Latn",  # Tektiteko
    "tte_Latn",  # Bwanabwana
    "tuc_Latn",  # Mutu
    "tue_Latn",  # Tuyuca
    "tuf_Latn",  # Central Tunebo
    "tuo_Latn",  # Tucano
    "tur_Latn",  # Turkish
    "tvk_Latn",  # Southeast Ambrym
    "twi_Latn",  # Twi
    "txq_Latn",  # Tii
    "txu_Latn",  # Kayapó
    "tzj_Latn",  # Tz'utujil
    "tzo_Latn",  # Tzotzil
    "ubr_Latn",  # Ubir
    "ubu_Latn",  # Umbu-Ungu
    "udu_Latn",  # Uduk
    "uig_Latn",  # Uighur
    "ukr_Cyrl",  # Ukrainian
    "uli_Latn",  # Ulithian
    "ulk_Latn",  # Meriam
    "upv_Latn",  # Uripiv-Wala-Rano-Atchin
    "ura_Latn",  # Urarina
    "urb_Latn",  # Kaapor
    "urd_Arab",  # Urdu
    "uri_Latn",  # Urim
    "urt_Latn",  # Urartian
    "urw_Latn",  # Sop
    "usa_Latn",  # Usarufa
    "usp_Latn",  # Uspanteco
    "uvh_Latn",  # Uri
    "uvl_Latn",  # Lote
    "vid_Latn",  # Vidunda
    "vie_Latn",  # Vietnamese
    "viv_Latn",  # Iduna
    "vmy_Latn",  # Mixtec, many Mixtec languages use Latin script, though there are multiple variants of Mixtec.
    "waj_Latn",  # Waffa
    "wal_Ethi",  # Wolaytta, uses the Ethiopic script
    "wap_Latn",  # Wapishana
    "wat_Latn",  # Kaninuwa
    "wbi_Latn",  # Vwanji
    "wbp_Latn",  # Warlpiri
    "wed_Latn",  # Wedau
    "wer_Latn",  # Weri
    "wim_Latn",  # Wik-Mungkan
    "wiu_Latn",  # Wiru
    "wiv_Latn",  # Vitu
    "wmt_Latn",  # Mwani
    "wmw_Latn",  # Mwani
    "wnc_Latn",  # Wantoat
    "wnu_Latn",  # Usan
    "wol_Latn",  # Wolof, primarily uses Latin script but also uses Arabic in some contexts
    "wos_Latn",  # Hanga Hundi
    "wrk_Latn",  # Garrwa
    "wro_Latn",  # Worrorra
    "wrs_Latn",  # Waris
    "wsk_Latn",  # Waskia
    "wuv_Latn",  # Wuvulu-Aua
    "xav_Latn",  # Xavante
    "xbi_Latn",  # Kombio
    "xed_Latn",  # Hdi
    "xla_Latn",  # Kamula
    "xnn_Latn",  # Northern Kankanay
    "xon_Latn",  # Konkomba
    "xsi_Latn",  # Sio
    "xtd_Latn",  # Diuxi-Tilantongo Mixtec
    "xtm_Latn",  # Magdalena Peñasco Mixtec
    "yaa_Latn",  # Yaminahua
    "yad_Latn",  # Yagua
    "yal_Latn",  # Yalunka
    "yap_Latn",  # Yapese
    "yaq_Latn",  # Yaqui
    "yby_Latn",  # Yawalapití
    "ycn_Latn",  # Yucuna
    "yka_Latn",  # Yakan
    "yle_Latn",  # Yele
    "yml_Latn",  # Iamalele
    "yon_Latn",  # Yongkom
    "yor_Latn",  # Yoruba, primarily uses the Latin script
    "yrb_Latn",  # Yareba
    "yre_Latn",  # Yaouré
    "yss_Latn",  # Yessan-Mayo
    "yuj_Latn",  # Karkar-Yuri
    "yut_Latn",  # Yopno
    "yuw_Latn",  # Yau (Morobe Province)
    "yva_Latn",  # Yawa
    "zaa_Latn",  # Sierra de Juárez Zapotec
    "zab_Latn",  # San Juan Guelavía Zapotec
    "zac_Latn",  # Ocotlán Zapotec
    "zad_Latn",  # Cajonos Zapotec
    "zai_Latn",  # Isthmus Zapotec
    "zaj_Latn",  # Zaramo
    "zam_Latn",  # Miahuatlán Zapotec
    "zao_Latn",  # Ozolotepec Zapotec
    "zap_Latn",  # Zapotec, uses Latin script across many varieties
    "zar_Latn",  # Rincón Zapotec
    "zas_Latn",  # Santo Domingo Albarradas Zapotec
    "zat_Latn",  # Tabaa Zapotec
    "zav_Latn",  # Yatzachi Zapotec
    "zaw_Latn",  # Mitla Zapotec
    "zca_Latn",  # Coatecas Altas Zapotec
    "zga_Latn",  # Ganza
    "zia_Latn",  # Zia
    "ziw_Latn",  # Zigula
    "zlm_Latn",  # Malay (individual language), Latin script
    "zos_Latn",  # Francisco León Zoque
    "zpc_Latn",  # Choapan Zapotec
    "zpl_Latn",  # Lachixío Zapotec
    "zpm_Latn",  # Mixtepec Zapotec
    "zpo_Latn",  # Amatlán Zapotec
    "zpq_Latn",  # Zoogocho Zapotec
    "zpu_Latn",  # Yalálag Zapotec
    "zpv_Latn",  # Chichicapan Zapotec
    "zpz_Latn",  # Texmelucan Zapotec
    "zsr_Latn",  # Southern Rincon Zapotec
    "ztq_Latn",  # Quioquitani-Quierí Zapotec
    "zty_Latn",  # Yatee Zapotec
    "zyp_Latn",  # Zyphe Chin
]

# train split because validation/test splits are extremely small in a lot of cases
_SPLIT = ["train"]

_N = 256


def extend_lang_pairs_english_centric() -> dict[str, list[str]]:
    # add all language pairs with english as source or target
    hf_lang_subset2isolang = {}
    for lang in _LANGUAGES:
        pair = f"eng_Latn-{lang}"
        hf_lang_subset2isolang[pair] = ["eng-Latn", lang.replace("_", "-")]
        pair = f"{lang}-eng_Latn"
        hf_lang_subset2isolang[pair] = [lang.replace("_", "-"), "eng-Latn"]
    return hf_lang_subset2isolang


_LANGUAGES_MAPPING = extend_lang_pairs_english_centric()


class BibleNLPBitextMining(AbsTaskBitextMining, MultilingualTask):
    metadata = TaskMetadata(
        name="BibleNLPBitextMining",
        dataset={
            "path": "davidstap/biblenlp-corpus-mmteb",
            "revision": "264a18480c529d9e922483839b4b9758e690b762",
            "split": f"train[:{_N}]",
            "trust_remote_code": True,
        },
        description="Partial Bible translations in 829 languages, aligned by verse.",
        reference="https://arxiv.org/abs/2304.09919",
        type="BitextMining",
        category="s2s",
        modalities=["text"],
        eval_splits=_SPLIT,
        eval_langs=_LANGUAGES_MAPPING,
        main_score="f1",
        # World English Bible (WEB) first draft 1997, finished 2020
        date=("1997-01-01", "2020-12-31"),
        domains=["Religious", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="created",
        descriptive_stats={
            "n_samples": {"train": _N},
            "avg_character_length": {"train": 120},
        },
        bibtex_citation="""@article{akerman2023ebible,
            title={The eBible Corpus: Data and Model Benchmarks for Bible Translation for Low-Resource Languages},
            author={Akerman, Vesa and Baines, David and Daspit, Damien and Hermjakob, Ulf and Jang, Taeho and Leong, Colin and Martin, Michael and Mathew, Joel and Robie, Jonathan and Schwarting, Marcus},
            journal={arXiv preprint arXiv:2304.09919},
            year={2023}
        }""",
    )

    def load_data(self, **kwargs: Any) -> None:
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return

        self.dataset = {}
        seen_pairs = []
        for lang in self.hf_subsets:
            # e.g. 'aai_Latn-eng_Latn' -> 'eng-aai'
            hf_lang_name = self._transform_lang_name_hf(lang)

            if hf_lang_name in seen_pairs:
                self.dataset[lang] = self.dataset[self._swap_substrings(lang)]
            else:
                dataset = datasets.load_dataset(
                    name=self._transform_lang_name_hf(lang),
                    **self.metadata_dict["dataset"],
                )
                self.dataset[lang] = datasets.DatasetDict({"train": dataset})
                seen_pairs.append(hf_lang_name)

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        # Convert to standard format
        for lang in self.hf_subsets:
            l1, l2 = (l.split("_")[0] for l in lang.split("-"))

            self.dataset[lang] = self.dataset[lang].rename_columns(
                {l1: "sentence1", l2: "sentence2"}
            )

    @staticmethod
    def _transform_lang_name_hf(lang):
        # Transform language name to match huggingface configuration
        langs = [l.split("_")[0] for l in lang.split("-")]
        if langs[1] == "eng":
            langs[0], langs[1] = langs[1], langs[0]
        return "-".join(langs)

    @staticmethod
    def _swap_substrings(string):
        substring1, substring2 = string.split("-")
        parts = string.split("-")
        if substring1 in parts and substring2 in parts:
            index1 = parts.index(substring1)
            index2 = parts.index(substring2)
            parts[index1], parts[index2] = parts[index2], parts[index1]
            new_string = "-".join(parts)
            return new_string
        else:
            return string



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class PatentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="PatentClassification",
        description="Classification Dataset of Patents and Abstract",
        dataset={
            "path": "ccdv/patent-classification",
            "revision": "2f38a1dfdecfacee0184d74eaeafd3c0fb49d2a6",
            "trust_remote_code": True,
        },
        reference="https://aclanthology.org/P19-1212.pdf",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2021-11-05", "2022-10-22"),
        domains=["Legal", "Written"],
        task_subtypes=["Topic classification"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{sharma-etal-2019-bigpatent,
            title = "{BIGPATENT}: A Large-Scale Dataset for Abstractive and Coherent Summarization",
            author = "Sharma, Eva  and
            Li, Chen  and
            Wang, Lu",
            editor = "Korhonen, Anna  and
            Traum, David  and
            M{\`a}rquez, Llu{\'\i}s",
            booktitle = "Proceedings of the 57th Annual Meeting of the Association for Computational Linguistics",
            month = jul,
            year = "2019",
            address = "Florence, Italy",
            publisher = "Association for Computational Linguistics",
            url = "https://aclanthology.org/P19-1212",
            doi = "10.18653/v1/P19-1212",
            pages = "2204--2213",
            abstract = "Most existing text summarization datasets are compiled from the news domain, where summaries have a flattened discourse structure. In such datasets, summary-worthy content often appears in the beginning of input articles. Moreover, large segments from input articles are present verbatim in their respective summaries. These issues impede the learning and evaluation of systems that can understand an article{'}s global content structure as well as produce abstractive summaries with high compression ratio. In this work, we present a novel dataset, BIGPATENT, consisting of 1.3 million records of U.S. patent documents along with human written abstractive summaries. Compared to existing summarization datasets, BIGPATENT has the following properties: i) summaries contain a richer discourse structure with more recurring entities, ii) salient content is evenly distributed in the input, and iii) lesser and shorter extractive fragments are present in the summaries. Finally, we train and evaluate baselines and popular learning models on BIGPATENT to shed light on new challenges and motivate future directions for summarization research.",
        }""",
        descriptive_stats={
            "n_samples": {"test": 5000},
            "avg_character_length": {"test": 18620.44},
        },
    )

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from __future__ import annotations

import logging

import pytest

import mteb
from mteb import MTEB
from mteb.encoder_interface import Encoder, EncoderWithQueryCorpusEncode
from mteb.model_meta import ModelMeta

logging.basicConfig(level=logging.INFO)


@pytest.mark.parametrize("task_name", ["BornholmBitextMining"])
@pytest.mark.parametrize("model_name", ["sentence-transformers/all-MiniLM-L6-v2"])
@pytest.mark.parametrize("model_revision", ["8b3219a92973c328a8e22fadcfa821b5dc75636a"])
def test_reproducibility_workflow(task_name: str, model_name: str, model_revision: str):
    """Test that a model and a task can be fetched and run in a reproducible fashion."""
    model_meta = mteb.get_model_meta(model_name, revision=model_revision)
    task = mteb.get_task(task_name)

    assert isinstance(model_meta, ModelMeta)
    assert isinstance(task, mteb.AbsTask)

    model = mteb.get_model(model_name, revision=model_revision)
    assert isinstance(model, (Encoder, EncoderWithQueryCorpusEncode))

    eval = MTEB(tasks=[task])
    eval.run(model, output_folder="tests/results", overwrite_results=True)



from __future__ import annotations

from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata

_EVAL_SPLIT = "test"


class CodeFeedbackST(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CodeFeedbackST",
        description="The dataset is a collection of user queries and assistant responses. The task is to retrieve the most relevant response for a given query.",
        reference="https://arxiv.org/abs/2407.02883",
        dataset={
            "path": "CoIR-Retrieval/codefeedback-st",
            "revision": "d213819e87aab9010628da8b73ab4eb337c89340",
        },
        type="Retrieval",
        category="p2p",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2019-12-31"),
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{li2024coircomprehensivebenchmarkcode,
        title={CoIR: A Comprehensive Benchmark for Code Information Retrieval Models},
        author={Xiangyang Li and Kuicai Dong and Yi Quan Lee and Wei Xia and Yichun Yin and Hao Zhang and Yong Liu and Yasheng Wang and Ruiming Tang},
        year={2024},
        eprint={2407.02883},
        archivePrefix={arXiv},
        primaryClass={cs.IR},
        url={https://arxiv.org/abs/2407.02883},
        }""",
        descriptive_stats={
            "n_samples": {
                _EVAL_SPLIT: 1000,
            },
            "avg_character_length": {
                "test": {
                    "average_document_length": 1521.3317148588733,
                    "average_query_length": 724.2441704465598,
                    "num_documents": 156526,
                    "num_queries": 31306,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskInstructionRetrieval import AbsTaskInstructionRetrieval


class Core17InstructionRetrieval(AbsTaskInstructionRetrieval):
    metadata = TaskMetadata(
        name="Core17InstructionRetrieval",
        description="Measuring retrieval instruction following ability on Core17 narratives.",
        reference="https://arxiv.org/abs/2403.15246",
        dataset={
            "path": "jhu-clsp/core17-instructions",
            "revision": "e39ff896cf3efbbdeeb950e6bd7c79f266995b07",
        },
        type="InstructionRetrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="p-MRR",
        date=("2023-08-01", "2024-04-01"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{weller2024followir,
      title={FollowIR: Evaluating and Teaching Information Retrieval Models to Follow Instructions}, 
      author={Orion Weller and Benjamin Chang and Sean MacAvaney and Kyle Lo and Arman Cohan and Benjamin Van Durme and Dawn Lawrie and Luca Soldaini},
      year={2024},
      eprint={2403.15246},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        descriptive_stats={
            "n_samples": {"eng": 19919 * 2},
            "test": {
                "num_docs": 19899,
                "num_queries": 20,
                "average_document_length": 2233.0329664807277,
                "average_query_length": 109.75,
                "average_instruction_length": 295.55,
                "average_changed_instruction_length": 355.2,
                "average_relevant_docs_per_query": 32.7,
                "average_top_ranked_per_query": 1000.0,
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class MacedonianTweetSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MacedonianTweetSentimentClassification",
        description="An Macedonian dataset for tweet sentiment classification.",
        reference="https://aclanthology.org/R15-1034/",
        dataset={
            "path": "isaacchung/macedonian-tweet-sentiment-classification",
            "revision": "957e075ba35e4417ba7837987fd7053a6533a1a2",
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        date=["2014-11-01", "2015-04-01"],
        eval_splits=["test"],
        eval_langs=["mkd-Cyrl"],
        main_score="accuracy",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{jovanoski-etal-2015-sentiment,
    title = "Sentiment Analysis in {T}witter for {M}acedonian",
    author = "Jovanoski, Dame  and
      Pachovski, Veno  and
      Nakov, Preslav",
    editor = "Mitkov, Ruslan  and
      Angelova, Galia  and
      Bontcheva, Kalina",
    booktitle = "Proceedings of the International Conference Recent Advances in Natural Language Processing",
    month = sep,
    year = "2015",
    address = "Hissar, Bulgaria",
    publisher = "INCOMA Ltd. Shoumen, BULGARIA",
    url = "https://aclanthology.org/R15-1034",
    pages = "249--257",
}""",
        descriptive_stats={
            "n_samples": {"test": 1139},
            "avg_character_length": {"test": 67.6},
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class ToxicConversationsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="ToxicConversationsClassification",
        description="Collection of comments from the Civil Comments platform together with annotations if the comment is toxic or not.",
        reference="https://www.kaggle.com/competitions/jigsaw-unintended-bias-in-toxicity-classification/overview",
        dataset={
            "path": "mteb/toxic_conversations_50k",
            "revision": "edfaf9da55d3dd50d43143d90c1ac476895ae6de",
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2017-01-01",
            "2018-12-31",
        ),  # Estimated range for the collection of comments
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{jigsaw-unintended-bias-in-toxicity-classification,
    author = {cjadams and Daniel Borkan and inversion and Jeffrey Sorensen and Lucas Dixon and Lucy Vasserman and nithum},
    title = {Jigsaw Unintended Bias in Toxicity Classification},
    publisher = {Kaggle},
    year = {2019},
    url = {https://kaggle.com/competitions/jigsaw-unintended-bias-in-toxicity-classification}
}""",
        descriptive_stats={
            "n_samples": {"test": 50000},
            "avg_character_length": {"test": 296.6},
        },
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["n_experiments"] = 10
        metadata_dict["samples_per_label"] = 16
        return metadata_dict

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from __future__ import annotations

import difflib

import mteb.benchmarks.benchmarks as benchmark_module
from mteb.benchmarks import Benchmark

BENCHMARK_REGISTRY = {
    inst.name: inst
    for nam, inst in benchmark_module.__dict__.items()
    if isinstance(inst, Benchmark)
}


def get_benchmark(
    benchmark_name: str,
) -> Benchmark:
    if benchmark_name not in BENCHMARK_REGISTRY:
        close_matches = difflib.get_close_matches(
            benchmark_name, BENCHMARK_REGISTRY.keys()
        )
        if close_matches:
            suggestion = f"KeyError: '{benchmark_name}' not found. Did you mean: {close_matches[0]}?"
        else:
            suggestion = f"KeyError: '{benchmark_name}' not found and no similar keys were found."
        raise KeyError(suggestion)
    return BENCHMARK_REGISTRY[benchmark_name]


def get_benchmarks(
    names: list[str] | None = None,
) -> list[Benchmark]:
    if names is None:
        names = list(BENCHMARK_REGISTRY.keys())
    return [BENCHMARK_REGISTRY[name] for name in names]



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class ArguAna(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="ArguAna",
        description="NFCorpus: A Full-Text Learning to Rank Dataset for Medical Information Retrieval",
        reference="http://argumentation.bplaced.net/arguana/data",
        dataset={
            "path": "mteb/arguana",
            "revision": "c22ab2a51041ffd869aaddef7af8d8215647e41a",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=["Medical", "Written"],
        task_subtypes=None,
        license="cc-by-sa-4.0",
        annotations_creators=None,
        dialect=[],
        sample_creation=None,
        bibtex_citation="""@inproceedings{boteva2016,
  author = {Boteva, Vera and Gholipour, Demian and Sokolov, Artem and Riezler, Stefan},
  title = {A Full-Text Learning to Rank Dataset for Medical Information Retrieval},
  journal = {Proceedings of the 38th European Conference on Information Retrieval},
  journal-abbrev = {ECIR},
  year = {2016},
  city = {Padova},
  country = {Italy},
  url = {http://www.cl.uni-heidelberg.de/~riezler/publications/papers/ECIR2016.pdf}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "average_document_length": 1029.2327645838136,
                    "average_query_length": 1192.7204836415362,
                    "num_documents": 8674,
                    "num_queries": 1406,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

import logging
from collections import defaultdict
from typing import Dict

import numpy as np

from mteb.load_results.load_results import MODEL_NAME, RESULTS, REVISION
from mteb.load_results.mteb_results import MTEBResults
from mteb.overview import get_task

logger = logging.getLogger(__name__)

AGGREGATION = Dict[MODEL_NAME, Dict[REVISION, Dict[str, float]]]


def mean(results: RESULTS) -> AGGREGATION:
    """Calculate the mean of the main score of the given results."""
    unique_tasks = set()
    for model, revisions in results.items():
        for revision, res in revisions.items():
            for result in res:
                unique_tasks.add(result.task_name)

    def _mean(model_name: str, rev: str, results: list[MTEBResults]) -> float:
        """Calculate the mean of the main score of the given results."""
        scores: list[float] = [result.get_score() for result in results]

        if len(scores) != len(unique_tasks):
            logger.warning(
                f"Model {model_name} revision {rev} has missing scores for some tasks"
            )

        if scores:
            return sum(scores) / len(unique_tasks)
        return np.nan

    return {
        model: {rev: {"mean": _mean(model, rev, res)} for rev, res in revs.items()}
        for model, revs in results.items()
    }


def task_category_weighted_mean(
    results: RESULTS,
) -> AGGREGATION:
    """Calculate the mean of the main score of the given results, weighted by the number of tasks of each type."""
    unique_tasks = set()
    task_types = defaultdict(set)
    for model, revisions in results.items():
        for revision, res in revisions.items():
            for result in res:
                task_name = result.task_name
                task_type = get_task(task_name).metadata.type
                unique_tasks.add(task_name)
                task_types[task_type].add(task_name)

    def _task_category_weighted_mean(
        model: str, rev: str, results: list[MTEBResults]
    ) -> dict[str, float]:
        """Calculate the mean of the main score of the given results, weighted by the number of tasks of each type."""
        _task_types = {task_type: [] for task_type in task_types.keys()}

        for result in results:
            task_name = result.task_name
            task_type = get_task(task_name).metadata.type
            _task_types[task_type].append(result.get_score())

        # mean pr task type then mean of means
        means = {}
        for task_type, scores in _task_types.items():
            if len(scores) != len(task_types[task_type]):
                logger.warning(
                    f"Model {model} revision {rev} has missing scores for some tasks of type {task_type}"
                )
            _mean = sum(scores) / len(task_types[task_type]) if scores else np.nan
            # means.append(_mean)
            means[f"mean ({task_type})"] = _mean

        _mean = sum(means.values()) / len(task_types)
        means["mean (weighted by task type)"] = _mean
        return means

    return {
        model: {
            rev: _task_category_weighted_mean(model, rev, res)
            for rev, res in revs.items()
        }
        for model, revs in results.items()
    }


def borda_count(
    results: RESULTS,
) -> AGGREGATION:
    """Calculate the Borda count of the given results.

    To handle ties, we use the [Tournament Borda Count method](https://en.wikipedia.org/wiki/Borda_count#Equal_ranks).
    This method assigns the average of the ranks that would have been assigned to the tied candidates to each of the tied candidates. So if two
    candidates would otherwise have gained 1 or 2 points (if not tied), they both gain 1.5 points.
    """
    # consider each model a candidate and each task a voter
    # each voter ranks the candidates

    n_candidates = sum(len(revs) for revs in results.values())
    candidate_scores = {
        model: {revision: 0.0 for revision in revisions}
        for model, revisions in results.items()
    }

    tasks = defaultdict(list)  # {task_name: [(model, revision, score), ...]}

    for model, revisions in results.items():
        for revision, task_results in revisions.items():
            for task_result in task_results:
                task_name = task_result.task_name
                score = task_result.get_score()
                tasks[task_name].append((model, revision, score))

    for task_name, task_results in tasks.items():
        task_results.sort(key=lambda x: x[2])
        # scores to assign to each candidate
        scores = list(range(0, n_candidates, 1))

        prev_result = None
        tied_group = []
        score = 0
        while task_results:
            _result = task_results.pop()

            if (prev_result is None) or (_result[2] == prev_result[2]):
                score += scores.pop()
                tied_group.append(_result)
            else:  # resolve point assignment
                for task_result in tied_group:
                    candidate_scores[task_result[0]][task_result[1]] += score / len(
                        tied_group
                    )
                tied_group = [_result]
                score = scores.pop()
            prev_result = _result

        # resolve last group
        for task_result in tied_group:
            candidate_scores[task_result[0]][task_result[1]] += score / len(tied_group)

    return {
        model: {rev: {"borda_count": score} for rev, score in revs.items()}
        for model, revs in candidate_scores.items()
    }


aggregation_methods = {
    "Mean (naïve)": mean,
    "Mean (weighted by task category)": task_category_weighted_mean,
    "Rank (Borda Count)": borda_count,
}



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGUAGES = {
    "pan": ["pan-Guru"],
    "bgc": ["bgc-Deva"],
    "mag": ["mag-Deva"],
    "bns": ["bns-Deva"],
    "kfq": ["kfg-Deva"],
    "noe": ["noe-Deva"],
    "bhb": ["bhb-Deva"],
    "bho": ["bho-Deva"],
    "gbm": ["gbm-Deva"],
    "mup": ["mup-Deva"],
    "anp": ["anp-Deva"],
    "hne": ["hne-Deva"],
    "bra": ["bra-Deva"],
    "raj": ["raj-Deva"],
    "awa": ["awa-Deva"],
    "guj": ["guj-Gujr"],
    "ben": ["ben-Beng"],
    "bhd": ["bhd-Deva"],
    "kfy": ["kfy-Deva"],
    "mar": ["mar-Deva"],
    "bjj": ["bjj-Deva"],
}


class HinDialectClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="HinDialectClassification",
        dataset={
            "path": "mlexplorer008/hin_dialect_classification",
            "revision": "944a44cf93932ce62b51e7c07d44d8cc03d6bcae",
        },
        description="HinDialect: 26 Hindi-related languages and dialects of the Indic Continuum in North India",
        reference="https://lindat.mff.cuni.cz/repository/xmlui/handle/11234/1-4839",
        category="s2s",
        modalities=["text"],
        type="Classification",
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="f1",
        date=("2010-01-01", "2023-01-01"),
        domains=["Social", "Spoken", "Written"],
        task_subtypes=["Language identification"],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
        @misc{11234/1-4839,
        title = {{HinDialect} 1.1: 26 Hindi-related languages and dialects of the Indic Continuum in North India},
        author = {Bafna, Niyati and {\v Z}abokrtsk{\'y}, Zden{\v e}k and Espa{\~n}a-Bonet, Cristina and van Genabith, Josef and Kumar, Lalit "Samyak Lalit" and Suman, Sharda and Shivay, Rahul},
        url = {http://hdl.handle.net/11234/1-4839},
        note = {{LINDAT}/{CLARIAH}-{CZ} digital library at the Institute of Formal and Applied Linguistics ({{\'U}FAL}), Faculty of Mathematics and Physics, Charles University},
        copyright = {Creative Commons - Attribution-{NonCommercial}-{ShareAlike} 4.0 International ({CC} {BY}-{NC}-{SA} 4.0)},
        year = {2022} }
        """,
        descriptive_stats={
            "n_samples": {"test": 1152},
            "avg_character_length": {"test": 583.82},
        },
    )

    def dataset_transform(self) -> None:
        self.dataset = self.dataset.rename_columns(
            {"folksong": "text", "language": "label"}
        )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class SIQA(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SIQA",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on SIQA.",
        reference="https://leaderboard.allenai.org/socialiqa/submissions/get-started",
        dataset={
            "path": "RAR-b/siqa",
            "revision": "4ed8415e9dc24060deefc84be59e2db0aacbadcc",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2019-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@article{xiao2024rar,
  title={RAR-b: Reasoning as Retrieval Benchmark},
  author={Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal={arXiv preprint arXiv:2404.06347},
  year={2024}
}
@article{sap2019socialiqa,
  title={Socialiqa: Commonsense reasoning about social interactions},
  author={Sap, Maarten and Rashkin, Hannah and Chen, Derek and LeBras, Ronan and Choi, Yejin},
  journal={arXiv preprint arXiv:1904.09728},
  year={2019}
}
""",
        descriptive_stats={
            "n_samples": {"test": 0},
            "avg_character_length": {
                "test": {
                    "average_document_length": 22.967085695044617,
                    "average_query_length": 127.75383828045035,
                    "num_documents": 71276,
                    "num_queries": 1954,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata

_EVAL_SPLIT = "test"


class CodeFeedbackMT(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CodeFeedbackMT",
        description="The dataset is a collection of user queries and assistant responses. The task is to retrieve the most relevant response for a given query.",
        reference="https://arxiv.org/abs/2402.14658",
        dataset={
            "path": "CoIR-Retrieval/codefeedback-mt",
            "revision": "b0f12fa0c0dd67f59c95a5c33d02aeeb4c398c5f",
        },
        type="Retrieval",
        category="p2p",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2019-12-31"),
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{zheng2024opencodeinterpreterintegratingcodegeneration,
              title={OpenCodeInterpreter: Integrating Code Generation with Execution and Refinement}, 
              author={Tianyu Zheng and Ge Zhang and Tianhao Shen and Xueling Liu and Bill Yuchen Lin and Jie Fu and Wenhu Chen and Xiang Yue},
              year={2024},
              eprint={2402.14658},
              archivePrefix={arXiv},
              primaryClass={cs.SE},
              url={https://arxiv.org/abs/2402.14658}, 
        }""",
        descriptive_stats={
            "n_samples": {
                _EVAL_SPLIT: 1000,
            },
            "avg_character_length": {
                "test": {
                    "average_document_length": 1467.879728243677,
                    "average_query_length": 4425.522256533855,
                    "num_documents": 66383,
                    "num_queries": 13277,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class FEVER(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="FEVER",
        dataset={
            "path": "mteb/fever",
            "revision": "bea83ef9e8fb933d90a2f1d5515737465d613e12",
        },
        description=(
            "FEVER (Fact Extraction and VERification) consists of 185,445 claims generated by altering sentences"
            + " extracted from Wikipedia and subsequently verified without knowledge of the sentence they were"
            + " derived from."
        ),
        reference="https://fever.ai/",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["train", "dev", "test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@inproceedings{thorne-etal-2018-fever,
    title = "{FEVER}: a Large-scale Dataset for Fact Extraction and {VER}ification",
    author = "Thorne, James  and
      Vlachos, Andreas  and
      Christodoulopoulos, Christos  and
      Mittal, Arpit",
    editor = "Walker, Marilyn  and
      Ji, Heng  and
      Stent, Amanda",
    booktitle = "Proceedings of the 2018 Conference of the North {A}merican Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 1 (Long Papers)",
    month = jun,
    year = "2018",
    address = "New Orleans, Louisiana",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/N18-1074",
    doi = "10.18653/v1/N18-1074",
    pages = "809--819",
    abstract = "In this paper we introduce a new publicly available dataset for verification against textual sources, FEVER: Fact Extraction and VERification. It consists of 185,445 claims generated by altering sentences extracted from Wikipedia and subsequently verified without knowledge of the sentence they were derived from. The claims are classified as Supported, Refuted or NotEnoughInfo by annotators achieving 0.6841 in Fleiss kappa. For the first two classes, the annotators also recorded the sentence(s) forming the necessary evidence for their judgment. To characterize the challenge of the dataset presented, we develop a pipeline approach and compare it to suitably designed oracles. The best accuracy we achieve on labeling a claim accompanied by the correct evidence is 31.87{\%}, while if we ignore the evidence we achieve 50.91{\%}. Thus we believe that FEVER is a challenging testbed that will help stimulate progress on claim verification against textual sources.",
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "train": {
                    "average_document_length": 538.2340070317589,
                    "average_query_length": 47.56034058828886,
                    "num_documents": 5416568,
                    "num_queries": 109810,
                    "average_relevant_docs_per_query": 1.2757034878426372,
                },
                "dev": {
                    "average_document_length": 538.2340070317589,
                    "average_query_length": 47.326282628262824,
                    "num_documents": 5416568,
                    "num_queries": 6666,
                    "average_relevant_docs_per_query": 1.211971197119712,
                },
                "test": {
                    "average_document_length": 538.2340070317589,
                    "average_query_length": 49.60546054605461,
                    "num_documents": 5416568,
                    "num_queries": 6666,
                    "average_relevant_docs_per_query": 1.1906690669066906,
                },
            },
        },
    )


class FEVERHardNegatives(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="FEVERHardNegatives",
        dataset={
            "path": "mteb/FEVER_test_top_250_only_w_correct-v2",
            "revision": "080c9ed6267b65029207906e815d44a9240bafca",
        },
        description=(
            "FEVER (Fact Extraction and VERification) consists of 185,445 claims generated by altering sentences"
            + " extracted from Wikipedia and subsequently verified without knowledge of the sentence they were"
            + " derived from. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct."
        ),
        reference="https://fever.ai/",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@inproceedings{thorne-etal-2018-fever,
    title = "{FEVER}: a Large-scale Dataset for Fact Extraction and {VER}ification",
    author = "Thorne, James  and
      Vlachos, Andreas  and
      Christodoulopoulos, Christos  and
      Mittal, Arpit",
    editor = "Walker, Marilyn  and
      Ji, Heng  and
      Stent, Amanda",
    booktitle = "Proceedings of the 2018 Conference of the North {A}merican Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 1 (Long Papers)",
    month = jun,
    year = "2018",
    address = "New Orleans, Louisiana",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/N18-1074",
    doi = "10.18653/v1/N18-1074",
    pages = "809--819",
    abstract = "In this paper we introduce a new publicly available dataset for verification against textual sources, FEVER: Fact Extraction and VERification. It consists of 185,445 claims generated by altering sentences extracted from Wikipedia and subsequently verified without knowledge of the sentence they were derived from. The claims are classified as Supported, Refuted or NotEnoughInfo by annotators achieving 0.6841 in Fleiss kappa. For the first two classes, the annotators also recorded the sentence(s) forming the necessary evidence for their judgment. To characterize the challenge of the dataset presented, we develop a pipeline approach and compare it to suitably designed oracles. The best accuracy we achieve on labeling a claim accompanied by the correct evidence is 31.87{\%}, while if we ignore the evidence we achieve 50.91{\%}. Thus we believe that FEVER is a challenging testbed that will help stimulate progress on claim verification against textual sources.",
}""",
        descriptive_stats={
            "n_samples": {"test": 1000},
            "avg_character_length": {
                "test": {
                    "average_document_length": 695.4370242764114,
                    "average_query_length": 49.62,
                    "num_documents": 163698,
                    "num_queries": 1000,
                    "average_relevant_docs_per_query": 1.171,
                }
            },
        },
    )



from __future__ import annotations

import datasets

from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import *

_LANGUAGES = {
    "ar": ["ara-Arab"],
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "es": ["spa-Latn"],
    "fr": ["fra-Latn"],
    "hi": ["hin-Deva"],
    "it": ["ita-Latn"],
    "ja": ["jpn-Jpan"],
    "ko": ["kor-Hang"],
    "pt": ["por-Latn"],
    "ru": ["rus-Cyrl"],
    "th": ["tha-Thai"],
    "zh": ["cmn-Hans"],
}


def load_mldr_data(
    path: str,
    langs: list,
    eval_splits: list,
    cache_dir: str = None,
    revision: str = None,
):
    corpus = {lang: {split: None for split in eval_splits} for lang in langs}
    queries = {lang: {split: None for split in eval_splits} for lang in langs}
    relevant_docs = {lang: {split: None for split in eval_splits} for lang in langs}

    for lang in langs:
        lang_corpus = datasets.load_dataset(
            path, f"corpus-{lang}", cache_dir=cache_dir, revision=revision
        )["corpus"]
        lang_corpus = {e["docid"]: {"text": e["text"]} for e in lang_corpus}
        lang_data = datasets.load_dataset(path, lang, cache_dir=cache_dir)
        for split in eval_splits:
            corpus[lang][split] = lang_corpus
            queries[lang][split] = {e["query_id"]: e["query"] for e in lang_data[split]}
            relevant_docs[lang][split] = {
                e["query_id"]: {e["positive_passages"][0]["docid"]: 1}
                for e in lang_data[split]
            }

    corpus = datasets.DatasetDict(corpus)
    queries = datasets.DatasetDict(queries)
    relevant_docs = datasets.DatasetDict(relevant_docs)
    return corpus, queries, relevant_docs


class MultiLongDocRetrieval(MultilingualTask, AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MultiLongDocRetrieval",
        description="""Multi Long Doc Retrieval (MLDR) 'is curated by the multilingual articles from Wikipedia, Wudao and mC4 (see Table 7), and NarrativeQA (Kocˇisky ́ et al., 2018; Gu ̈nther et al., 2023), which is only for English.' (Chen et al., 2024). 
        It is constructed by sampling lengthy articles from Wikipedia, Wudao and mC4 datasets and randomly choose paragraphs from them. Then we use GPT-3.5 to generate questions based on these paragraphs. The generated question and the sampled article constitute a new text pair to the dataset.""",
        reference="https://arxiv.org/abs/2402.03216",  # also: https://huggingface.co/datasets/Shitao/MLDR
        dataset={
            "path": "Shitao/MLDR",
            "revision": "d67138e705d963e346253a80e59676ddb418810a",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev", "test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_10",
        date=(
            "2000-01-01",
            "2024-12-31",
        ),  # Not found in the paper, guessed using the paper's publication date and constituent datasets
        domains=[
            "Encyclopaedic",
            "Written",
            "Web",
            "Non-fiction",
            "Fiction",
        ],  # narrativeqa, wikipedia, wudao, mC4
        task_subtypes=[],
        license="mit",
        annotations_creators="LM-generated",  # gpt-3.5
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{bge-m3,
      title={BGE M3-Embedding: Multi-Lingual, Multi-Functionality, Multi-Granularity Text Embeddings Through Self-Knowledge Distillation},
      author={Jianlv Chen and Shitao Xiao and Peitian Zhang and Kun Luo and Defu Lian and Zheng Liu},
      year={2024},
      eprint={2402.03216},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "ar": {
                        "average_document_length": 29234.48153016958,
                        "average_query_length": 69.27,
                        "num_documents": 7607,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "de": {
                        "average_document_length": 33771.2111,
                        "average_query_length": 153.63,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "en": {
                        "average_document_length": 13332.76764,
                        "average_query_length": 81.22,
                        "num_documents": 200000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "es": {
                        "average_document_length": 36567.1736990891,
                        "average_query_length": 123.11,
                        "num_documents": 9551,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "fr": {
                        "average_document_length": 36009.4934,
                        "average_query_length": 142.165,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "hi": {
                        "average_document_length": 18688.50788229112,
                        "average_query_length": 77.995,
                        "num_documents": 3806,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "it": {
                        "average_document_length": 36633.9969,
                        "average_query_length": 99.615,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ja": {
                        "average_document_length": 14480.7508,
                        "average_query_length": 61.625,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ko": {
                        "average_document_length": 13813.441224093263,
                        "average_query_length": 58.845,
                        "num_documents": 6176,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "pt": {
                        "average_document_length": 32127.576952351956,
                        "average_query_length": 122.275,
                        "num_documents": 6569,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ru": {
                        "average_document_length": 35934.8756,
                        "average_query_length": 87.875,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "th": {
                        "average_document_length": 25993.2696,
                        "average_query_length": 107.81,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "zh": {
                        "average_document_length": 6039.059725,
                        "average_query_length": 26.79,
                        "num_documents": 200000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                },
                "test": {
                    "ar": {
                        "average_document_length": 29234.48153016958,
                        "average_query_length": 75.77,
                        "num_documents": 7607,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "de": {
                        "average_document_length": 33771.2111,
                        "average_query_length": 123.65,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "en": {
                        "average_document_length": 13332.76764,
                        "average_query_length": 81.33,
                        "num_documents": 200000,
                        "num_queries": 800,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "es": {
                        "average_document_length": 36567.1736990891,
                        "average_query_length": 131.985,
                        "num_documents": 9551,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "fr": {
                        "average_document_length": 36009.4934,
                        "average_query_length": 149.795,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "hi": {
                        "average_document_length": 18688.50788229112,
                        "average_query_length": 103.76,
                        "num_documents": 3806,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "it": {
                        "average_document_length": 36633.9969,
                        "average_query_length": 114.595,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ja": {
                        "average_document_length": 14480.7508,
                        "average_query_length": 55.73,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ko": {
                        "average_document_length": 13813.441224093263,
                        "average_query_length": 58.72,
                        "num_documents": 6176,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "pt": {
                        "average_document_length": 32127.576952351956,
                        "average_query_length": 113.455,
                        "num_documents": 6569,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ru": {
                        "average_document_length": 35934.8756,
                        "average_query_length": 94.87,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "th": {
                        "average_document_length": 25993.2696,
                        "average_query_length": 97.99,
                        "num_documents": 10000,
                        "num_queries": 200,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "zh": {
                        "average_document_length": 6039.059725,
                        "average_query_length": 24.70875,
                        "num_documents": 200000,
                        "num_queries": 800,
                        "average_relevant_docs_per_query": 1.0,
                    },
                },
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_mldr_data(
            path=self.metadata_dict["dataset"]["path"],
            langs=self.metadata.eval_langs,
            eval_splits=self.metadata_dict["eval_splits"],
            cache_dir=kwargs.get("cache_dir", None),
            revision=self.metadata_dict["dataset"]["revision"],
        )
        self.data_loaded = True



from __future__ import annotations

import logging
from typing import Sequence

import numpy as np
import torch

from mteb.encoder_interface import Encoder, PromptType

logger = logging.getLogger(__name__)


def model_encode(
    sentences: Sequence[str],
    *,
    model: Encoder,
    task_name: str | None,
    prompt_type: PromptType | None = None,
    **kwargs,
) -> np.ndarray:
    """A wrapper function around the model.encode method that handles the prompt_name argument and standardizes the output to a numpy array.

    Args:
        sentences: The sentences to encode
        model: The model to use for encoding
        task_name: The task name to use for building the encoding prompt
        prompt_type: The prompt type (e.g. "query" | "passage") to use for building the encoding prompt
        **kwargs: Additional arguments to pass to the model.encode method
    """
    ## The order of priorities are:
    # 1. Composed prompt of task name + prompt type
    # 2. Specific task prompt
    # 3. Specific prompt type

    if hasattr(model, "prompts"):
        # check if prompts is an empty dict
        if not model.prompts:  # type: ignore
            logger.info(
                "Model does not support prompts. Removing prompt_name argument."
            )
            kwargs.pop("prompt_name", None)

        if task_name and prompt_type and f"{task_name}-{ prompt_type}" in model.prompts:
            kwargs["prompt_name"] = f"{task_name}-{ prompt_type}"
        elif task_name and task_name in model.prompts:
            kwargs["prompt_name"] = task_name
        elif prompt_type and prompt_type in model.prompts:
            kwargs["prompt_name"] = prompt_type

        else:  # type: ignore
            logger.info(
                "No combination of task name and prompt type was found in model prompts. Removing prompt_name argument."
            )
            kwargs.pop("prompt_name", None)
    else:
        kwargs["prompt_name"] = task_name

    logger.info(f"Encoding {len(sentences)} sentences.")

    embeddings = model.encode(sentences, **kwargs)
    if isinstance(embeddings, torch.Tensor):
        embeddings = embeddings.cpu().detach().float()

    return np.asarray(embeddings)
